@@ -1,7 +1,3 @@
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
@@ -11,38 +7,38 @@
 #include "../toxcore/net_crypto.h"
 #include "check_compat.h"
 
-static void rand_bytes(uint8_t *b, size_t blen)
+static void rand_bytes(const Random *rng, uint8_t *b, size_t blen)
 {
     size_t i;
 
     for (i = 0; i < blen; i++) {
-        b[i] = random_u08();
+        b[i] = random_u08(rng);
     }
 }
 
 // These test vectors are from libsodium's test suite
 
-static const unsigned char alicesk[32] = {
+static const uint8_t alicesk[32] = {
     0x77, 0x07, 0x6d, 0x0a, 0x73, 0x18, 0xa5, 0x7d,
     0x3c, 0x16, 0xc1, 0x72, 0x51, 0xb2, 0x66, 0x45,
     0xdf, 0x4c, 0x2f, 0x87, 0xeb, 0xc0, 0x99, 0x2a,
     0xb1, 0x77, 0xfb, 0xa5, 0x1d, 0xb9, 0x2c, 0x2a
 };
 
-static const unsigned char bobpk[32] = {
+static const uint8_t bobpk[32] = {
     0xde, 0x9e, 0xdb, 0x7d, 0x7b, 0x7d, 0xc1, 0xb4,
     0xd3, 0x5b, 0x61, 0xc2, 0xec, 0xe4, 0x35, 0x37,
     0x3f, 0x83, 0x43, 0xc8, 0x5b, 0x78, 0x67, 0x4d,
     0xad, 0xfc, 0x7e, 0x14, 0x6f, 0x88, 0x2b, 0x4f
 };
 
-static const unsigned char test_nonce[24] = {
+static const uint8_t test_nonce[24] = {
     0x69, 0x69, 0x6e, 0xe9, 0x55, 0xb6, 0x2b, 0x73,
     0xcd, 0x62, 0xbd, 0xa8, 0x75, 0xfc, 0x73, 0xd6,
     0x82, 0x19, 0xe0, 0x03, 0x6b, 0x7a, 0x0b, 0x37
 };
 
-static const unsigned char test_m[131] = {
+static const uint8_t test_m[131] = {
     0xbe, 0x07, 0x5f, 0xc5, 0x3c, 0x81, 0xf2, 0xd5,
     0xcf, 0x14, 0x13, 0x16, 0xeb, 0xeb, 0x0c, 0x7b,
     0x52, 0x28, 0xc5, 0x2a, 0x4c, 0x62, 0xcb, 0xd4,
@@ -62,7 +58,7 @@ static const unsigned char test_m[131] = {
     0x5e, 0x07, 0x05
 };
 
-static const unsigned char test_c[147] = {
+static const uint8_t test_c[147] = {
     0xf3, 0xff, 0xc7, 0x70, 0x3f, 0x94, 0x00, 0xe5,
     0x2a, 0x7d, 0xfb, 0x4b, 0x3d, 0x33, 0x05, 0xd9,
     0x8e, 0x99, 0x3b, 0x9f, 0x48, 0x68, 0x12, 0x73,
@@ -84,92 +80,88 @@ static const unsigned char test_c[147] = {
     0xe3, 0x55, 0xa5
 };
 
-START_TEST(test_known)
+static void test_known(void)
 {
-    unsigned char c[147];
-    unsigned char m[131];
+    uint8_t c[147];
+    uint8_t m[131];
     uint16_t clen, mlen;
 
-    ck_assert_msg(sizeof(c) == sizeof(m) + CRYPTO_MAC_SIZE * sizeof(unsigned char),
+    ck_assert_msg(sizeof(c) == sizeof(m) + CRYPTO_MAC_SIZE * sizeof(uint8_t),
                   "cyphertext should be CRYPTO_MAC_SIZE bytes longer than plaintext");
     ck_assert_msg(sizeof(test_c) == sizeof(c), "sanity check failed");
     ck_assert_msg(sizeof(test_m) == sizeof(m), "sanity check failed");
 
-    clen = encrypt_data(bobpk, alicesk, test_nonce, test_m, sizeof(test_m) / sizeof(unsigned char), c);
+    clen = encrypt_data(bobpk, alicesk, test_nonce, test_m, sizeof(test_m) / sizeof(uint8_t), c);
 
     ck_assert_msg(memcmp(test_c, c, sizeof(c)) == 0, "cyphertext doesn't match test vector");
-    ck_assert_msg(clen == sizeof(c) / sizeof(unsigned char), "wrong ciphertext length");
+    ck_assert_msg(clen == sizeof(c) / sizeof(uint8_t), "wrong ciphertext length");
 
-    mlen = decrypt_data(bobpk, alicesk, test_nonce, test_c, sizeof(test_c) / sizeof(unsigned char), m);
+    mlen = decrypt_data(bobpk, alicesk, test_nonce, test_c, sizeof(test_c) / sizeof(uint8_t), m);
 
     ck_assert_msg(memcmp(test_m, m, sizeof(m)) == 0, "decrypted text doesn't match test vector");
-    ck_assert_msg(mlen == sizeof(m) / sizeof(unsigned char), "wrong plaintext length");
+    ck_assert_msg(mlen == sizeof(m) / sizeof(uint8_t), "wrong plaintext length");
 }
-END_TEST
 
-START_TEST(test_fast_known)
+static void test_fast_known(void)
 {
-    unsigned char k[CRYPTO_SHARED_KEY_SIZE];
-    unsigned char c[147];
-    unsigned char m[131];
+    uint8_t k[CRYPTO_SHARED_KEY_SIZE];
+    uint8_t c[147];
+    uint8_t m[131];
     uint16_t clen, mlen;
 
     encrypt_precompute(bobpk, alicesk, k);
 
-    ck_assert_msg(sizeof(c) == sizeof(m) + CRYPTO_MAC_SIZE * sizeof(unsigned char),
+    ck_assert_msg(sizeof(c) == sizeof(m) + CRYPTO_MAC_SIZE * sizeof(uint8_t),
                   "cyphertext should be CRYPTO_MAC_SIZE bytes longer than plaintext");
     ck_assert_msg(sizeof(test_c) == sizeof(c), "sanity check failed");
     ck_assert_msg(sizeof(test_m) == sizeof(m), "sanity check failed");
 
-    clen = encrypt_data_symmetric(k, test_nonce, test_m, sizeof(test_m) / sizeof(unsigned char), c);
+    clen = encrypt_data_symmetric(k, test_nonce, test_m, sizeof(test_m) / sizeof(uint8_t), c);
 
     ck_assert_msg(memcmp(test_c, c, sizeof(c)) == 0, "cyphertext doesn't match test vector");
-    ck_assert_msg(clen == sizeof(c) / sizeof(unsigned char), "wrong ciphertext length");
+    ck_assert_msg(clen == sizeof(c) / sizeof(uint8_t), "wrong ciphertext length");
 
-    mlen = decrypt_data_symmetric(k, test_nonce, test_c, sizeof(test_c) / sizeof(unsigned char), m);
+    mlen = decrypt_data_symmetric(k, test_nonce, test_c, sizeof(test_c) / sizeof(uint8_t), m);
 
     ck_assert_msg(memcmp(test_m, m, sizeof(m)) == 0, "decrypted text doesn't match test vector");
-    ck_assert_msg(mlen == sizeof(m) / sizeof(unsigned char), "wrong plaintext length");
+    ck_assert_msg(mlen == sizeof(m) / sizeof(uint8_t), "wrong plaintext length");
 }
-END_TEST
 
-START_TEST(test_endtoend)
+static void test_endtoend(void)
 {
-    unsigned char pk1[CRYPTO_PUBLIC_KEY_SIZE];
-    unsigned char sk1[CRYPTO_SECRET_KEY_SIZE];
-    unsigned char pk2[CRYPTO_PUBLIC_KEY_SIZE];
-    unsigned char sk2[CRYPTO_SECRET_KEY_SIZE];
-    unsigned char k1[CRYPTO_SHARED_KEY_SIZE];
-    unsigned char k2[CRYPTO_SHARED_KEY_SIZE];
-
-    unsigned char n[CRYPTO_NONCE_SIZE];
-
-    unsigned char m[500];
-    unsigned char c1[sizeof(m) + CRYPTO_MAC_SIZE];
-    unsigned char c2[sizeof(m) + CRYPTO_MAC_SIZE];
-    unsigned char c3[sizeof(m) + CRYPTO_MAC_SIZE];
-    unsigned char c4[sizeof(m) + CRYPTO_MAC_SIZE];
-    unsigned char m1[sizeof(m)];
-    unsigned char m2[sizeof(m)];
-    unsigned char m3[sizeof(m)];
-    unsigned char m4[sizeof(m)];
-
-    uint16_t mlen;
-    uint16_t c1len, c2len, c3len, c4len;
-    uint16_t m1len, m2len, m3len, m4len;
-
-    uint8_t testno;
+    const Random *rng = system_random();
+    ck_assert(rng != nullptr);
 
     // Test 100 random messages and keypairs
-    for (testno = 0; testno < 100; testno++) {
-        //Generate random message (random length from 100 to 500)
-        mlen = (random_u32() % 400) + 100;
-        rand_bytes(m, mlen);
-        rand_bytes(n, CRYPTO_NONCE_SIZE);
+    for (uint8_t testno = 0; testno < 100; testno++) {
+        uint8_t pk1[CRYPTO_PUBLIC_KEY_SIZE];
+        uint8_t sk1[CRYPTO_SECRET_KEY_SIZE];
+        uint8_t pk2[CRYPTO_PUBLIC_KEY_SIZE];
+        uint8_t sk2[CRYPTO_SECRET_KEY_SIZE];
+        uint8_t k1[CRYPTO_SHARED_KEY_SIZE];
+        uint8_t k2[CRYPTO_SHARED_KEY_SIZE];
+
+        uint8_t n[CRYPTO_NONCE_SIZE];
+
+        enum { M_SIZE = 50 };
+        uint8_t m[M_SIZE];
+        uint8_t c1[sizeof(m) + CRYPTO_MAC_SIZE];
+        uint8_t c2[sizeof(m) + CRYPTO_MAC_SIZE];
+        uint8_t c3[sizeof(m) + CRYPTO_MAC_SIZE];
+        uint8_t c4[sizeof(m) + CRYPTO_MAC_SIZE];
+        uint8_t m1[sizeof(m)];
+        uint8_t m2[sizeof(m)];
+        uint8_t m3[sizeof(m)];
+        uint8_t m4[sizeof(m)];
+
+        //Generate random message (random length from 10 to 50)
+        const uint16_t mlen = (random_u32(rng) % (M_SIZE - 10)) + 10;
+        rand_bytes(rng, m, mlen);
+        rand_bytes(rng, n, CRYPTO_NONCE_SIZE);
 
         //Generate keypairs
-        crypto_new_keypair(pk1, sk1);
-        crypto_new_keypair(pk2, sk2);
+        crypto_new_keypair(rng, pk1, sk1);
+        crypto_new_keypair(rng, pk2, sk2);
 
         //Precompute shared keys
         encrypt_precompute(pk2, sk1, k1);
@@ -178,10 +170,10 @@ START_TEST(test_endtoend)
         ck_assert_msg(memcmp(k1, k2, CRYPTO_SHARED_KEY_SIZE) == 0, "encrypt_precompute: bad");
 
         //Encrypt all four ways
-        c1len = encrypt_data(pk2, sk1, n, m, mlen, c1);
-        c2len = encrypt_data(pk1, sk2, n, m, mlen, c2);
-        c3len = encrypt_data_symmetric(k1, n, m, mlen, c3);
-        c4len = encrypt_data_symmetric(k2, n, m, mlen, c4);
+        const uint16_t c1len = encrypt_data(pk2, sk1, n, m, mlen, c1);
+        const uint16_t c2len = encrypt_data(pk1, sk2, n, m, mlen, c2);
+        const uint16_t c3len = encrypt_data_symmetric(k1, n, m, mlen, c3);
+        const uint16_t c4len = encrypt_data_symmetric(k2, n, m, mlen, c4);
 
         ck_assert_msg(c1len == c2len && c1len == c3len && c1len == c4len, "cyphertext lengths differ");
         ck_assert_msg(c1len == mlen + (uint16_t)CRYPTO_MAC_SIZE, "wrong cyphertext length");
@@ -189,10 +181,10 @@ START_TEST(test_endtoend)
                       && memcmp(c1, c4, c1len) == 0, "crypertexts differ");
 
         //Decrypt all four ways
-        m1len = decrypt_data(pk2, sk1, n, c1, c1len, m1);
-        m2len = decrypt_data(pk1, sk2, n, c1, c1len, m2);
-        m3len = decrypt_data_symmetric(k1, n, c1, c1len, m3);
-        m4len = decrypt_data_symmetric(k2, n, c1, c1len, m4);
+        const uint16_t m1len = decrypt_data(pk2, sk1, n, c1, c1len, m1);
+        const uint16_t m2len = decrypt_data(pk1, sk2, n, c1, c1len, m2);
+        const uint16_t m3len = decrypt_data_symmetric(k1, n, c1, c1len, m3);
+        const uint16_t m4len = decrypt_data_symmetric(k2, n, c1, c1len, m4);
 
         ck_assert_msg(m1len == m2len && m1len == m3len && m1len == m4len, "decrypted text lengths differ");
         ck_assert_msg(m1len == mlen, "wrong decrypted text length");
@@ -201,74 +193,85 @@ START_TEST(test_endtoend)
         ck_assert_msg(memcmp(m1, m, mlen) == 0, "wrong decrypted text");
     }
 }
-END_TEST
 
-START_TEST(test_large_data)
+static void test_large_data(void)
 {
-    unsigned char k[CRYPTO_SHARED_KEY_SIZE];
+    const Random *rng = system_random();
+    ck_assert(rng != nullptr);
+    uint8_t k[CRYPTO_SHARED_KEY_SIZE];
+    uint8_t n[CRYPTO_NONCE_SIZE];
 
-    unsigned char n[CRYPTO_NONCE_SIZE];
+    const size_t m1_size = MAX_CRYPTO_PACKET_SIZE - CRYPTO_MAC_SIZE;
+    uint8_t *m1 = (uint8_t *)malloc(m1_size);
+    uint8_t *c1 = (uint8_t *)malloc(m1_size + CRYPTO_MAC_SIZE);
+    uint8_t *m1prime = (uint8_t *)malloc(m1_size);
 
-    unsigned char m1[MAX_CRYPTO_PACKET_SIZE - CRYPTO_MAC_SIZE];
-    unsigned char c1[sizeof(m1) + CRYPTO_MAC_SIZE];
-    unsigned char m1prime[sizeof(m1)];
+    const size_t m2_size = MAX_CRYPTO_PACKET_SIZE - CRYPTO_MAC_SIZE;
+    uint8_t *m2 = (uint8_t *)malloc(m2_size);
+    uint8_t *c2 = (uint8_t *)malloc(m2_size + CRYPTO_MAC_SIZE);
 
-    unsigned char m2[MAX_CRYPTO_PACKET_SIZE];
-    unsigned char c2[sizeof(m2) + CRYPTO_MAC_SIZE];
-
-    uint16_t c1len, c2len;
-    uint16_t m1plen;
+    ck_assert(m1 != nullptr && c1 != nullptr && m1prime != nullptr && m2 != nullptr && c2 != nullptr);
 
     //Generate random messages
-    rand_bytes(m1, sizeof(m1));
-    rand_bytes(m2, sizeof(m2));
-    rand_bytes(n, CRYPTO_NONCE_SIZE);
+    rand_bytes(rng, m1, m1_size);
+    rand_bytes(rng, m2, m2_size);
+    rand_bytes(rng, n, CRYPTO_NONCE_SIZE);
 
     //Generate key
-    rand_bytes(k, CRYPTO_SHARED_KEY_SIZE);
+    rand_bytes(rng, k, CRYPTO_SHARED_KEY_SIZE);
 
-    c1len = encrypt_data_symmetric(k, n, m1, sizeof(m1), c1);
-    c2len = encrypt_data_symmetric(k, n, m2, sizeof(m2), c2);
+    const uint16_t c1len = encrypt_data_symmetric(k, n, m1, m1_size, c1);
+    const uint16_t c2len = encrypt_data_symmetric(k, n, m2, m2_size, c2);
 
-    ck_assert_msg(c1len == sizeof(m1) + CRYPTO_MAC_SIZE, "could not encrypt");
-    ck_assert_msg(c2len == sizeof(m2) + CRYPTO_MAC_SIZE, "could not encrypt");
+    ck_assert_msg(c1len == m1_size + CRYPTO_MAC_SIZE, "could not encrypt");
+    ck_assert_msg(c2len == m2_size + CRYPTO_MAC_SIZE, "could not encrypt");
 
-    m1plen = decrypt_data_symmetric(k, n, c1, c1len, m1prime);
+    const uint16_t m1plen = decrypt_data_symmetric(k, n, c1, c1len, m1prime);
 
-    ck_assert_msg(m1plen == sizeof(m1), "decrypted text lengths differ");
-    ck_assert_msg(memcmp(m1prime, m1, sizeof(m1)) == 0, "decrypted texts differ");
+    ck_assert_msg(m1plen == m1_size, "decrypted text lengths differ");
+    ck_assert_msg(memcmp(m1prime, m1, m1_size) == 0, "decrypted texts differ");
+
+    free(c2);
+    free(m2);
+    free(m1prime);
+    free(c1);
+    free(m1);
 }
-END_TEST
 
-START_TEST(test_large_data_symmetric)
+static void test_large_data_symmetric(void)
 {
-    unsigned char k[CRYPTO_SYMMETRIC_KEY_SIZE];
+    const Random *rng = system_random();
+    ck_assert(rng != nullptr);
+    uint8_t k[CRYPTO_SYMMETRIC_KEY_SIZE];
 
-    unsigned char n[CRYPTO_NONCE_SIZE];
+    uint8_t n[CRYPTO_NONCE_SIZE];
 
-    unsigned char m1[16 * 16 * 16];
-    unsigned char c1[sizeof(m1) + CRYPTO_MAC_SIZE];
-    unsigned char m1prime[sizeof(m1)];
+    const size_t m1_size = 16 * 16 * 16;
+    uint8_t *m1 = (uint8_t *)malloc(m1_size);
+    uint8_t *c1 = (uint8_t *)malloc(m1_size + CRYPTO_MAC_SIZE);
+    uint8_t *m1prime = (uint8_t *)malloc(m1_size);
 
-    uint16_t c1len;
-    uint16_t m1plen;
+    ck_assert(m1 != nullptr && c1 != nullptr && m1prime != nullptr);
 
     //Generate random messages
-    rand_bytes(m1, sizeof(m1));
-    rand_bytes(n, CRYPTO_NONCE_SIZE);
+    rand_bytes(rng, m1, m1_size);
+    rand_bytes(rng, n, CRYPTO_NONCE_SIZE);
 
     //Generate key
-    new_symmetric_key(k);
+    new_symmetric_key(rng, k);
 
-    c1len = encrypt_data_symmetric(k, n, m1, sizeof(m1), c1);
-    ck_assert_msg(c1len == sizeof(m1) + CRYPTO_MAC_SIZE, "could not encrypt data");
+    const uint16_t c1len = encrypt_data_symmetric(k, n, m1, m1_size, c1);
+    ck_assert_msg(c1len == m1_size + CRYPTO_MAC_SIZE, "could not encrypt data");
 
-    m1plen = decrypt_data_symmetric(k, n, c1, c1len, m1prime);
+    const uint16_t m1plen = decrypt_data_symmetric(k, n, c1, c1len, m1prime);
 
-    ck_assert_msg(m1plen == sizeof(m1), "decrypted text lengths differ");
-    ck_assert_msg(memcmp(m1prime, m1, sizeof(m1)) == 0, "decrypted texts differ");
+    ck_assert_msg(m1plen == m1_size, "decrypted text lengths differ");
+    ck_assert_msg(memcmp(m1prime, m1, m1_size) == 0, "decrypted texts differ");
+
+    free(m1prime);
+    free(c1);
+    free(m1);
 }
-END_TEST
 
 static void increment_nonce_number_cmp(uint8_t *nonce, uint32_t num)
 {
@@ -291,14 +294,17 @@ static void increment_nonce_number_cmp(uint8_t *nonce, uint32_t num)
     memcpy(nonce + (CRYPTO_NONCE_SIZE - sizeof(num2)), &num2, sizeof(num2));
 }
 
-START_TEST(test_increment_nonce)
+static void test_increment_nonce(void)
 {
+    const Random *rng = system_random();
+    ck_assert(rng != nullptr);
+
     uint32_t i;
 
     uint8_t n[CRYPTO_NONCE_SIZE];
 
     for (i = 0; i < CRYPTO_NONCE_SIZE; ++i) {
-        n[i] = random_u08();
+        n[i] = random_u08(rng);
     }
 
     uint8_t n1[CRYPTO_NONCE_SIZE];
@@ -312,15 +318,14 @@ START_TEST(test_increment_nonce)
     }
 
     for (i = 0; i < (1 << 18); ++i) {
-        const uint32_t r = random_u32();
+        const uint32_t r = random_u32(rng);
         increment_nonce_number_cmp(n, r);
         increment_nonce_number(n1, r);
         ck_assert_msg(memcmp(n, n1, CRYPTO_NONCE_SIZE) == 0, "Bad increment_nonce_number function");
     }
 }
-END_TEST
 
-START_TEST(test_memzero)
+static void test_memzero(void)
 {
     uint8_t src[sizeof(test_c)];
     memcpy(src, test_c, sizeof(test_c));
@@ -332,35 +337,18 @@ START_TEST(test_memzero)
         ck_assert_msg(src[i] == 0, "Memory is not zeroed");
     }
 }
-END_TEST
-
-static Suite *crypto_suite(void)
-{
-    Suite *s = suite_create("Crypto");
-
-    DEFTESTCASE(known);
-    DEFTESTCASE(fast_known);
-    DEFTESTCASE_SLOW(endtoend, 15); /* waiting up to 15 seconds */
-    DEFTESTCASE(large_data);
-    DEFTESTCASE(large_data_symmetric);
-    DEFTESTCASE_SLOW(increment_nonce, 20);
-    DEFTESTCASE(memzero);
-
-    return s;
-}
 
 int main(void)
 {
     setvbuf(stdout, nullptr, _IONBF, 0);
 
-    Suite *crypto = crypto_suite();
-    SRunner *test_runner = srunner_create(crypto);
-    uint8_t number_failed = 0;
+    test_known();
+    test_fast_known();
+    test_endtoend(); /* waiting up to 15 seconds */
+    test_large_data();
+    test_large_data_symmetric();
+    test_increment_nonce();
+    test_memzero();
 
-    srunner_run_all(test_runner, CK_NORMAL);
-    number_failed = srunner_ntests_failed(test_runner);
-
-    srunner_free(test_runner);
-
-    return number_failed;
+    return 0;
 }
