@@ -1,7 +1,3 @@
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include <stdlib.h>
 #include <string.h>
 
@@ -11,6 +7,7 @@
 #include "../toxcore/onion_announce.h"
 #include "../toxcore/onion_client.h"
 #include "../toxcore/util.h"
+#include "auto_test_support.h"
 #include "check_compat.h"
 
 #ifndef USE_IPV6
@@ -21,30 +18,30 @@ static inline IP get_loopback(void)
 {
     IP ip;
 #if USE_IPV6
-    ip.family = net_family_ipv6;
+    ip.family = net_family_ipv6();
     ip.ip.v6 = get_ip6_loopback();
 #else
-    ip.family = net_family_ipv4;
+    ip.family = net_family_ipv4();
     ip.ip.v4 = get_ip4_loopback();
 #endif
     return ip;
 }
-static void do_onion(Onion *onion)
+static void do_onion(Mono_Time *mono_time, Onion *onion)
 {
-    mono_time_update(onion->mono_time);
+    mono_time_update(mono_time);
 
     networking_poll(onion->net, nullptr);
     do_dht(onion->dht);
 }
 
 static int handled_test_1;
-static int handle_test_1(void *object, IP_Port source, const uint8_t *packet, uint16_t length, void *userdata)
+static int handle_test_1(void *object, const IP_Port *source, const uint8_t *packet, uint16_t length, void *userdata)
 {
     Onion *onion = (Onion *)object;
 
     const char req_message[] = "Install Gentoo";
     uint8_t req_packet[1 + sizeof(req_message)];
-    req_packet[0] = NET_PACKET_ANNOUNCE_REQUEST;
+    req_packet[0] = NET_PACKET_ANNOUNCE_REQUEST_OLD;
     memcpy(req_packet + 1, req_message, sizeof(req_message));
 
     if (memcmp(packet, req_packet, sizeof(req_packet)) != 0) {
@@ -53,7 +50,7 @@ static int handle_test_1(void *object, IP_Port source, const uint8_t *packet, ui
 
     const char res_message[] = "install gentoo";
     uint8_t res_packet[1 + sizeof(res_message)];
-    res_packet[0] = NET_PACKET_ANNOUNCE_RESPONSE;
+    res_packet[0] = NET_PACKET_ANNOUNCE_RESPONSE_OLD;
     memcpy(res_packet + 1, res_message, sizeof(res_message));
 
     if (send_onion_response(onion->net, source, res_packet, sizeof(res_packet),
@@ -66,11 +63,11 @@ static int handle_test_1(void *object, IP_Port source, const uint8_t *packet, ui
 }
 
 static int handled_test_2;
-static int handle_test_2(void *object, IP_Port source, const uint8_t *packet, uint16_t length, void *userdata)
+static int handle_test_2(void *object, const IP_Port *source, const uint8_t *packet, uint16_t length, void *userdata)
 {
     const char res_message[] = "install gentoo";
     uint8_t res_packet[1 + sizeof(res_message)];
-    res_packet[0] = NET_PACKET_ANNOUNCE_RESPONSE;
+    res_packet[0] = NET_PACKET_ANNOUNCE_RESPONSE_OLD;
     memcpy(res_packet + 1, res_message, sizeof(res_message));
 
     if (length != sizeof(res_packet)) {
@@ -100,7 +97,7 @@ static uint8_t sb_data[ONION_ANNOUNCE_SENDBACK_DATA_LENGTH];
 static int handled_test_3;
 static uint8_t test_3_pub_key[CRYPTO_PUBLIC_KEY_SIZE];
 static uint8_t test_3_ping_id[CRYPTO_SHA256_SIZE];
-static int handle_test_3(void *object, IP_Port source, const uint8_t *packet, uint16_t length, void *userdata)
+static int handle_test_3(void *object, const IP_Port *source, const uint8_t *packet, uint16_t length, void *userdata)
 {
     Onion *onion = (Onion *)object;
 
@@ -137,7 +134,7 @@ static int handle_test_3(void *object, IP_Port source, const uint8_t *packet, ui
 
 static uint8_t nonce[CRYPTO_NONCE_SIZE];
 static int handled_test_4;
-static int handle_test_4(void *object, IP_Port source, const uint8_t *packet, uint16_t length, void *userdata)
+static int handle_test_4(void *object, const IP_Port *source, const uint8_t *packet, uint16_t length, void *userdata)
 {
     Onion *onion = (Onion *)object;
 
@@ -167,22 +164,47 @@ static int handle_test_4(void *object, IP_Port source, const uint8_t *packet, ui
     return 0;
 }
 
+/** Create and send a onion packet.
+ *
+ * Use Onion_Path path to send data of length to dest.
+ * Maximum length of data is ONION_MAX_DATA_SIZE.
+ */
+static void send_onion_packet(const Networking_Core *net, const Random *rng, const Onion_Path *path, const IP_Port *dest, const uint8_t *data, uint16_t length)
+{
+    uint8_t packet[ONION_MAX_PACKET_SIZE];
+    const int len = create_onion_packet(rng, packet, sizeof(packet), path, dest, data, length);
+    ck_assert_msg(len != -1, "failed to create onion packet");
+    ck_assert_msg(sendpacket(net, &path->ip_port1, packet, len) == len, "failed to send onion packet");
+}
+
+/** Initialize networking.
+ * Added for reverse compatibility with old new_networking calls.
+ */
+static Networking_Core *new_networking(const Logger *log, const Network *ns, const IP *ip, uint16_t port)
+{
+    return new_networking_ex(log, ns, ip, port, port + (TOX_PORTRANGE_TO - TOX_PORTRANGE_FROM), nullptr);
+}
+
 static void test_basic(void)
 {
     uint32_t index[] = { 1, 2, 3 };
+    const Network *ns = system_network();
+
     Logger *log1 = logger_new();
     logger_callback_log(log1, (logger_cb *)print_debug_log, nullptr, &index[0]);
     Logger *log2 = logger_new();
     logger_callback_log(log2, (logger_cb *)print_debug_log, nullptr, &index[1]);
 
-    Mono_Time *mono_time1 = mono_time_new();
-    Mono_Time *mono_time2 = mono_time_new();
+    const Random *rng = system_random();
+    ck_assert(rng != nullptr);
+    Mono_Time *mono_time1 = mono_time_new(nullptr, nullptr);
+    Mono_Time *mono_time2 = mono_time_new(nullptr, nullptr);
 
     IP ip = get_loopback();
-    Onion *onion1 = new_onion(mono_time1, new_dht(log1, mono_time1, new_networking(log1, ip, 36567), true));
-    Onion *onion2 = new_onion(mono_time2, new_dht(log2, mono_time2, new_networking(log2, ip, 36568), true));
+    Onion *onion1 = new_onion(log1, mono_time1, rng, new_dht(log1, rng, ns, mono_time1, new_networking(log1, ns, &ip, 36567), true, false));
+    Onion *onion2 = new_onion(log2, mono_time2, rng, new_dht(log2, rng, ns, mono_time2, new_networking(log2, ns, &ip, 36568), true, false));
     ck_assert_msg((onion1 != nullptr) && (onion2 != nullptr), "Onion failed initializing.");
-    networking_registerhandler(onion2->net, NET_PACKET_ANNOUNCE_REQUEST, &handle_test_1, onion2);
+    networking_registerhandler(onion2->net, NET_PACKET_ANNOUNCE_REQUEST_OLD, &handle_test_1, onion2);
 
     IP_Port on1 = {ip, net_port(onion1->net)};
     Node_format n1;
@@ -196,7 +218,7 @@ static void test_basic(void)
 
     const char req_message[] = "Install Gentoo";
     uint8_t req_packet[1 + sizeof(req_message)];
-    req_packet[0] = NET_PACKET_ANNOUNCE_REQUEST;
+    req_packet[0] = NET_PACKET_ANNOUNCE_REQUEST_OLD;
     memcpy(req_packet + 1, req_message, sizeof(req_message));
 
     Node_format nodes[4];
@@ -205,55 +227,54 @@ static void test_basic(void)
     nodes[2] = n1;
     nodes[3] = n2;
     Onion_Path path;
-    create_onion_path(onion1->dht, &path, nodes);
-    int ret = send_onion_packet(onion1->net, &path, nodes[3].ip_port, req_packet, sizeof(req_packet));
-    ck_assert_msg(ret == 0, "Failed to create/send onion packet.");
+    create_onion_path(rng, onion1->dht, &path, nodes);
+    send_onion_packet(onion1->net, rng, &path, &nodes[3].ip_port, req_packet, sizeof(req_packet));
 
     handled_test_1 = 0;
 
     do {
-        do_onion(onion1);
-        do_onion(onion2);
+        do_onion(mono_time1, onion1);
+        do_onion(mono_time2, onion2);
     } while (handled_test_1 == 0);
 
-    networking_registerhandler(onion1->net, NET_PACKET_ANNOUNCE_RESPONSE, &handle_test_2, onion1);
+    networking_registerhandler(onion1->net, NET_PACKET_ANNOUNCE_RESPONSE_OLD, &handle_test_2, onion1);
     handled_test_2 = 0;
 
     do {
-        do_onion(onion1);
-        do_onion(onion2);
+        do_onion(mono_time1, onion1);
+        do_onion(mono_time2, onion2);
     } while (handled_test_2 == 0);
 
-    Onion_Announce *onion1_a = new_onion_announce(mono_time1, onion1->dht);
-    Onion_Announce *onion2_a = new_onion_announce(mono_time2, onion2->dht);
-    networking_registerhandler(onion1->net, NET_PACKET_ANNOUNCE_RESPONSE, &handle_test_3, onion1);
+    Onion_Announce *onion1_a = new_onion_announce(log1, rng, mono_time1, onion1->dht);
+    Onion_Announce *onion2_a = new_onion_announce(log2, rng, mono_time2, onion2->dht);
+    networking_registerhandler(onion1->net, NET_PACKET_ANNOUNCE_RESPONSE_OLD, &handle_test_3, onion1);
     ck_assert_msg((onion1_a != nullptr) && (onion2_a != nullptr), "Onion_Announce failed initializing.");
     uint8_t zeroes[64] = {0};
-    random_bytes(sb_data, sizeof(sb_data));
+    random_bytes(rng, sb_data, sizeof(sb_data));
     uint64_t s;
     memcpy(&s, sb_data, sizeof(uint64_t));
     memcpy(test_3_pub_key, nodes[3].public_key, CRYPTO_PUBLIC_KEY_SIZE);
-    ret = send_announce_request(onion1->net, &path, nodes[3],
-                                dht_get_self_public_key(onion1->dht),
-                                dht_get_self_secret_key(onion1->dht),
-                                zeroes,
-                                dht_get_self_public_key(onion1->dht),
-                                dht_get_self_public_key(onion1->dht), s);
+    int ret = send_announce_request(onion1->net, rng, &path, &nodes[3],
+                                    dht_get_self_public_key(onion1->dht),
+                                    dht_get_self_secret_key(onion1->dht),
+                                    zeroes,
+                                    dht_get_self_public_key(onion1->dht),
+                                    dht_get_self_public_key(onion1->dht), s);
     ck_assert_msg(ret == 0, "Failed to create/send onion announce_request packet.");
     handled_test_3 = 0;
 
     do {
-        do_onion(onion1);
-        do_onion(onion2);
+        do_onion(mono_time1, onion1);
+        do_onion(mono_time2, onion2);
         c_sleep(50);
     } while (handled_test_3 == 0);
 
-    random_bytes(sb_data, sizeof(sb_data));
+    random_bytes(rng, sb_data, sizeof(sb_data));
     memcpy(&s, sb_data, sizeof(uint64_t));
     memcpy(onion_announce_entry_public_key(onion2_a, 1), dht_get_self_public_key(onion2->dht), CRYPTO_PUBLIC_KEY_SIZE);
     onion_announce_entry_set_time(onion2_a, 1, mono_time_get(mono_time2));
     networking_registerhandler(onion1->net, NET_PACKET_ONION_DATA_RESPONSE, &handle_test_4, onion1);
-    send_announce_request(onion1->net, &path, nodes[3],
+    send_announce_request(onion1->net, rng, &path, &nodes[3],
                           dht_get_self_public_key(onion1->dht),
                           dht_get_self_secret_key(onion1->dht),
                           test_3_ping_id,
@@ -261,8 +282,8 @@ static void test_basic(void)
                           dht_get_self_public_key(onion1->dht), s);
 
     do {
-        do_onion(onion1);
-        do_onion(onion2);
+        do_onion(mono_time1, onion1);
+        do_onion(mono_time2, onion2);
         c_sleep(50);
     } while (memcmp(onion_announce_entry_public_key(onion2_a, ONION_ANNOUNCE_MAX_ENTRIES - 2),
                     dht_get_self_public_key(onion1->dht),
@@ -272,13 +293,13 @@ static void test_basic(void)
     Logger *log3 = logger_new();
     logger_callback_log(log3, (logger_cb *)print_debug_log, nullptr, &index[2]);
 
-    Mono_Time *mono_time3 = mono_time_new();
+    Mono_Time *mono_time3 = mono_time_new(nullptr, nullptr);
 
-    Onion *onion3 = new_onion(mono_time3, new_dht(log3, mono_time3, new_networking(log3, ip, 36569), true));
+    Onion *onion3 = new_onion(log3, mono_time3, rng, new_dht(log3, rng, ns, mono_time3, new_networking(log3, ns, &ip, 36569), true, false));
     ck_assert_msg((onion3 != nullptr), "Onion failed initializing.");
 
-    random_nonce(nonce);
-    ret = send_data_request(onion3->net, &path, nodes[3].ip_port,
+    random_nonce(rng, nonce);
+    ret = send_data_request(onion3->net, rng, &path, &nodes[3].ip_port,
                             dht_get_self_public_key(onion1->dht),
                             dht_get_self_public_key(onion1->dht),
                             nonce, (const uint8_t *)"Install gentoo", sizeof("Install gentoo"));
@@ -286,8 +307,8 @@ static void test_basic(void)
     handled_test_4 = 0;
 
     do {
-        do_onion(onion1);
-        do_onion(onion2);
+        do_onion(mono_time1, onion1);
+        do_onion(mono_time2, onion2);
         c_sleep(50);
     } while (handled_test_4 == 0);
 
@@ -339,10 +360,11 @@ typedef struct {
     Onion_Client *onion_c;
 } Onions;
 
-static Onions *new_onions(uint16_t port, uint32_t *index)
+static Onions *new_onions(const Random *rng, uint16_t port, uint32_t *index)
 {
     IP ip = get_loopback();
     ip.ip.v6.uint8[15] = 1;
+    const Network *ns = system_network();
     Onions *on = (Onions *)malloc(sizeof(Onions));
 
     if (!on) {
@@ -358,7 +380,7 @@ static Onions *new_onions(uint16_t port, uint32_t *index)
 
     logger_callback_log(on->log, (logger_cb *)print_debug_log, nullptr, index);
 
-    on->mono_time = mono_time_new();
+    on->mono_time = mono_time_new(nullptr, nullptr);
 
     if (!on->mono_time) {
         logger_kill(on->log);
@@ -366,7 +388,7 @@ static Onions *new_onions(uint16_t port, uint32_t *index)
         return nullptr;
     }
 
-    Networking_Core *net = new_networking(on->log, ip, port);
+    Networking_Core *net = new_networking(on->log, ns, &ip, port);
 
     if (!net) {
         mono_time_free(on->mono_time);
@@ -375,7 +397,7 @@ static Onions *new_onions(uint16_t port, uint32_t *index)
         return nullptr;
     }
 
-    DHT *dht = new_dht(on->log, on->mono_time, net, true);
+    DHT *dht = new_dht(on->log, rng, ns, on->mono_time, net, true, false);
 
     if (!dht) {
         kill_networking(net);
@@ -385,7 +407,7 @@ static Onions *new_onions(uint16_t port, uint32_t *index)
         return nullptr;
     }
 
-    on->onion = new_onion(on->mono_time, dht);
+    on->onion = new_onion(on->log, on->mono_time, rng, dht);
 
     if (!on->onion) {
         kill_dht(dht);
@@ -396,7 +418,7 @@ static Onions *new_onions(uint16_t port, uint32_t *index)
         return nullptr;
     }
 
-    on->onion_a = new_onion_announce(on->mono_time, dht);
+    on->onion_a = new_onion_announce(on->log, rng, on->mono_time, dht);
 
     if (!on->onion_a) {
         kill_onion(on->onion);
@@ -409,7 +431,7 @@ static Onions *new_onions(uint16_t port, uint32_t *index)
     }
 
     TCP_Proxy_Info inf = {{{{0}}}};
-    on->onion_c = new_onion_client(on->log, on->mono_time, new_net_crypto(on->log, on->mono_time, dht, &inf));
+    on->onion_c = new_onion_client(on->log, rng, on->mono_time, new_net_crypto(on->log, rng, ns, on->mono_time, dht, &inf));
 
     if (!on->onion_c) {
         kill_onion_announce(on->onion_a);
@@ -455,7 +477,7 @@ static void kill_onions(Onions *on)
 #define NUM_LAST 37
 
 static bool first_ip, last_ip;
-static void dht_ip_callback(void *object, int32_t number, IP_Port ip_port)
+static void dht_ip_callback(void *object, int32_t number, const IP_Port *ip_port)
 {
     if (NUM_FIRST == number) {
         first_ip = 1;
@@ -512,10 +534,12 @@ static void test_announce(void)
     uint32_t i, j;
     uint32_t index[NUM_ONIONS];
     Onions *onions[NUM_ONIONS];
+    const Random *rng = system_random();
+    ck_assert(rng != nullptr);
 
     for (i = 0; i < NUM_ONIONS; ++i) {
         index[i] = i + 1;
-        onions[i] = new_onions(i + 36655, &index[i]);
+        onions[i] = new_onions(rng, i + 36655, &index[i]);
         ck_assert_msg(onions[i] != nullptr, "Failed to create onions. %u", i);
     }
 
@@ -523,11 +547,11 @@ static void test_announce(void)
 
     for (i = 3; i < NUM_ONIONS; ++i) {
         IP_Port ip_port = {ip, net_port(onions[i - 1]->onion->net)};
-        dht_bootstrap(onions[i]->onion->dht, ip_port, dht_get_self_public_key(onions[i - 1]->onion->dht));
+        dht_bootstrap(onions[i]->onion->dht, &ip_port, dht_get_self_public_key(onions[i - 1]->onion->dht));
         IP_Port ip_port1 = {ip, net_port(onions[i - 2]->onion->net)};
-        dht_bootstrap(onions[i]->onion->dht, ip_port1, dht_get_self_public_key(onions[i - 2]->onion->dht));
+        dht_bootstrap(onions[i]->onion->dht, &ip_port1, dht_get_self_public_key(onions[i - 2]->onion->dht));
         IP_Port ip_port2 = {ip, net_port(onions[i - 3]->onion->net)};
-        dht_bootstrap(onions[i]->onion->dht, ip_port2, dht_get_self_public_key(onions[i - 3]->onion->dht));
+        dht_bootstrap(onions[i]->onion->dht, &ip_port2, dht_get_self_public_key(onions[i - 3]->onion->dht));
     }
 
     uint32_t connected = 0;
