@@ -1,7 +1,3 @@
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -15,6 +11,7 @@
 #include "../toxcore/logger.h"
 #include "../toxcore/tox.h"
 #include "../toxcore/util.h"
+#include "auto_test_support.h"
 #include "check_compat.h"
 
 #define TEST_REGULAR_AV 1
@@ -61,7 +58,7 @@ static void t_toxav_call_cb(ToxAV *av, uint32_t friend_number, bool audio_enable
 
 static void t_toxav_call_state_cb(ToxAV *av, uint32_t friend_number, uint32_t state, void *user_data)
 {
-    printf("Handling CALL STATE callback: %d\n", state);
+    printf("Handling CALL STATE callback: %u\n", state);
     ((CallControl *)user_data)->state = state;
 }
 
@@ -104,6 +101,12 @@ static void iterate_tox(Tox *bootstrap, Tox *Alice, Tox *Bob)
     tox_iterate(Bob, nullptr);
 }
 
+static bool toxav_audio_send_frame_helper(ToxAV *av, uint32_t friend_number, Toxav_Err_Send_Frame *error)
+{
+    static const int16_t PCM[960] = {0};
+    return toxav_audio_send_frame(av, 0, PCM, 960, 1, 48000, nullptr);
+}
+
 static void regular_call_flow(
     Tox *Alice, Tox *Bob, Tox *bootstrap,
     ToxAV *AliceAV, ToxAV *BobAV,
@@ -116,10 +119,7 @@ static void regular_call_flow(
     Toxav_Err_Call call_err;
     toxav_call(AliceAV, 0, a_br, v_br, &call_err);
 
-    if (call_err != TOXAV_ERR_CALL_OK) {
-        printf("toxav_call failed: %d\n", call_err);
-        ck_assert(0);
-    }
+    ck_assert_msg(call_err == TOXAV_ERR_CALL_OK, "toxav_call failed: %d\n", call_err);
 
     const time_t start_time = time(nullptr);
 
@@ -128,10 +128,7 @@ static void regular_call_flow(
             Toxav_Err_Answer answer_err;
             toxav_answer(BobAV, 0, a_br, v_br, &answer_err);
 
-            if (answer_err != TOXAV_ERR_ANSWER_OK) {
-                printf("toxav_answer failed: %d\n", answer_err);
-                ck_assert(0);
-            }
+            ck_assert_msg(answer_err == TOXAV_ERR_ANSWER_OK, "toxav_answer failed: %d\n", answer_err);
 
             BobCC->incoming = false;
         } else { /* TODO(mannol): rtp */
@@ -140,10 +137,7 @@ static void regular_call_flow(
                 Toxav_Err_Call_Control cc_err;
                 toxav_call_control(AliceAV, 0, TOXAV_CALL_CONTROL_CANCEL, &cc_err);
 
-                if (cc_err != TOXAV_ERR_CALL_CONTROL_OK) {
-                    printf("toxav_call_control failed: %d\n", cc_err);
-                    ck_assert(0);
-                }
+                ck_assert_msg(cc_err == TOXAV_ERR_CALL_CONTROL_OK, "toxav_call_control failed: %d\n", cc_err);
             }
         }
 
@@ -264,10 +258,7 @@ static void test_av_flows(void)
             Toxav_Err_Call rc;
             toxav_call(AliceAV, 0, 48, 0, &rc);
 
-            if (rc != TOXAV_ERR_CALL_OK) {
-                printf("toxav_call failed: %d\n", rc);
-                ck_assert(0);
-            }
+            ck_assert_msg(rc == TOXAV_ERR_CALL_OK, "toxav_call failed: %d\n", rc);
         }
 
         do {
@@ -279,10 +270,7 @@ static void test_av_flows(void)
             Toxav_Err_Call_Control rc;
             toxav_call_control(BobAV, 0, TOXAV_CALL_CONTROL_CANCEL, &rc);
 
-            if (rc != TOXAV_ERR_CALL_CONTROL_OK) {
-                printf("toxav_call_control failed: %d\n", rc);
-                ck_assert(0);
-            }
+            ck_assert_msg(rc == TOXAV_ERR_CALL_CONTROL_OK, "toxav_call_control failed: %d\n", rc);
         }
 
         do {
@@ -302,10 +290,7 @@ static void test_av_flows(void)
             Toxav_Err_Call rc;
             toxav_call(AliceAV, 0, 48, 0, &rc);
 
-            if (rc != TOXAV_ERR_CALL_OK) {
-                printf("toxav_call failed: %d\n", rc);
-                ck_assert(0);
-            }
+            ck_assert_msg(rc == TOXAV_ERR_CALL_OK, "toxav_call failed: %d\n", rc);
         }
 
         do {
@@ -317,10 +302,7 @@ static void test_av_flows(void)
             Toxav_Err_Call_Control rc;
             toxav_call_control(AliceAV, 0, TOXAV_CALL_CONTROL_CANCEL, &rc);
 
-            if (rc != TOXAV_ERR_CALL_CONTROL_OK) {
-                printf("toxav_call_control failed: %d\n", rc);
-                ck_assert(0);
-            }
+            ck_assert_msg(rc == TOXAV_ERR_CALL_CONTROL_OK, "toxav_call_control failed: %d\n", rc);
         }
 
         /* Alice will not receive end state */
@@ -342,10 +324,7 @@ static void test_av_flows(void)
             Toxav_Err_Call rc;
             toxav_call(AliceAV, 0, 48, 1000, &rc);
 
-            if (rc != TOXAV_ERR_CALL_OK) {
-                printf("toxav_call failed: %d\n", rc);
-                ck_assert(0);
-            }
+            ck_assert_msg(rc == TOXAV_ERR_CALL_OK, "toxav_call failed: %d\n", rc);
         }
 
         do {
@@ -364,10 +343,7 @@ static void test_av_flows(void)
             Toxav_Err_Answer rc;
             toxav_answer(BobAV, 0, 48, 4000, &rc);
 
-            if (rc != TOXAV_ERR_ANSWER_OK) {
-                printf("toxav_answer failed: %d\n", rc);
-                ck_assert(0);
-            }
+            ck_assert_msg(rc == TOXAV_ERR_ANSWER_OK, "toxav_answer failed: %d\n", rc);
         }
 
         iterate_tox(bootstrap, Alice, Bob);
@@ -409,10 +385,7 @@ static void test_av_flows(void)
             Toxav_Err_Call_Control rc;
             toxav_call_control(AliceAV, 0, TOXAV_CALL_CONTROL_CANCEL, &rc);
 
-            if (rc != TOXAV_ERR_CALL_CONTROL_OK) {
-                printf("toxav_call_control failed: %d\n", rc);
-                ck_assert(0);
-            }
+            ck_assert_msg(rc == TOXAV_ERR_CALL_CONTROL_OK, "toxav_call_control failed: %d\n", rc);
         }
 
         iterate_tox(bootstrap, Alice, Bob);
@@ -432,10 +405,7 @@ static void test_av_flows(void)
             Toxav_Err_Call rc;
             toxav_call(AliceAV, 0, 48, 0, &rc);
 
-            if (rc != TOXAV_ERR_CALL_OK) {
-                printf("toxav_call failed: %d\n", rc);
-                ck_assert(0);
-            }
+            ck_assert_msg(rc == TOXAV_ERR_CALL_OK, "toxav_call failed: %d\n", rc);
         }
 
         do {
@@ -446,10 +416,7 @@ static void test_av_flows(void)
             Toxav_Err_Answer rc;
             toxav_answer(BobAV, 0, 48, 0, &rc);
 
-            if (rc != TOXAV_ERR_ANSWER_OK) {
-                printf("toxav_answer failed: %d\n", rc);
-                ck_assert(0);
-            }
+            ck_assert_msg(rc == TOXAV_ERR_ANSWER_OK, "toxav_answer failed: %d\n", rc);
         }
 
         iterate_tox(bootstrap, Alice, Bob);
@@ -477,10 +444,7 @@ static void test_av_flows(void)
             Toxav_Err_Call_Control rc;
             toxav_call_control(AliceAV, 0, TOXAV_CALL_CONTROL_CANCEL, &rc);
 
-            if (rc != TOXAV_ERR_CALL_CONTROL_OK) {
-                printf("toxav_call_control failed: %d\n", rc);
-                ck_assert(0);
-            }
+            ck_assert_msg(rc == TOXAV_ERR_CALL_CONTROL_OK, "toxav_call_control failed: %d\n", rc);
         }
 
         iterate_tox(bootstrap, Alice, Bob);
@@ -500,10 +464,7 @@ static void test_av_flows(void)
             Toxav_Err_Call rc;
             toxav_call(AliceAV, 0, 48, 0, &rc);
 
-            if (rc != TOXAV_ERR_CALL_OK) {
-                printf("toxav_call failed: %d\n", rc);
-                ck_assert(0);
-            }
+            ck_assert_msg(rc == TOXAV_ERR_CALL_OK, "toxav_call failed: %d\n", rc);
         }
 
         do {
@@ -514,33 +475,25 @@ static void test_av_flows(void)
             Toxav_Err_Answer rc;
             toxav_answer(BobAV, 0, 48, 0, &rc);
 
-            if (rc != TOXAV_ERR_ANSWER_OK) {
-                printf("toxav_answer failed: %d\n", rc);
-                ck_assert(0);
-            }
+            ck_assert_msg(rc == TOXAV_ERR_ANSWER_OK, "toxav_answer failed: %d\n", rc);
         }
 
-        int16_t PCM[5670];
-
         iterate_tox(bootstrap, Alice, Bob);
         ck_assert_call_control(AliceAV, 0, TOXAV_CALL_CONTROL_PAUSE);
         iterate_tox(bootstrap, Alice, Bob);
-        ck_assert(!toxav_audio_send_frame(AliceAV, 0, PCM, 960, 1, 48000, nullptr));
-        ck_assert(!toxav_audio_send_frame(BobAV, 0, PCM, 960, 1, 48000, nullptr));
+        ck_assert(!toxav_audio_send_frame_helper(AliceAV, 0, nullptr));
+        ck_assert(!toxav_audio_send_frame_helper(BobAV, 0, nullptr));
         ck_assert_call_control(AliceAV, 0, TOXAV_CALL_CONTROL_RESUME);
         iterate_tox(bootstrap, Alice, Bob);
-        ck_assert(toxav_audio_send_frame(AliceAV, 0, PCM, 960, 1, 48000, nullptr));
-        ck_assert(toxav_audio_send_frame(BobAV, 0, PCM, 960, 1, 48000, nullptr));
+        ck_assert(toxav_audio_send_frame_helper(AliceAV, 0, nullptr));
+        ck_assert(toxav_audio_send_frame_helper(BobAV, 0, nullptr));
         iterate_tox(bootstrap, Alice, Bob);
 
         {
             Toxav_Err_Call_Control rc;
             toxav_call_control(AliceAV, 0, TOXAV_CALL_CONTROL_CANCEL, &rc);
 
-            if (rc != TOXAV_ERR_CALL_CONTROL_OK) {
-                printf("toxav_call_control failed: %d\n", rc);
-                ck_assert(0);
-            }
+            ck_assert_msg(rc == TOXAV_ERR_CALL_CONTROL_OK, "toxav_call_control failed: %d\n", rc);
         }
 
         iterate_tox(bootstrap, Alice, Bob);
