@@ -1,7 +1,3 @@
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -20,7 +16,9 @@
 #include "../toxcore/crypto_core.h"
 #include "../toxcore/logger.h"
 #include "../toxcore/tox.h"
+#include "../toxcore/tox_struct.h"
 #include "../toxcore/util.h"
+#include "auto_test_support.h"
 #include "check_compat.h"
 
 typedef struct {
@@ -47,7 +45,7 @@ static void t_toxav_call_cb(ToxAV *av, uint32_t friend_number, bool audio_enable
 
 static void t_toxav_call_state_cb(ToxAV *av, uint32_t friend_number, uint32_t state, void *user_data)
 {
-    printf("Handling CALL STATE callback: %d %p\n", state, (void *)av);
+    printf("Handling CALL STATE callback: %u %p\n", state, (void *)av);
     ((CallControl *)user_data)[friend_number].state = state;
 }
 
@@ -72,7 +70,7 @@ static void t_accept_friend_request_cb(Tox *m, const uint8_t *public_key, const
                                        void *userdata)
 {
     if (length == 7 && memcmp("gentoo", data, 7) == 0) {
-        ck_assert(tox_friend_add_norequest(m, public_key, nullptr) != (uint32_t) ~0);
+        ck_assert(tox_friend_add_norequest(m, public_key, nullptr) != (uint32_t) -1);
     }
 }
 
@@ -99,39 +97,8 @@ static void *call_thread(void *pd)
 {
     ToxAV *AliceAV = ((thread_data *) pd)->AliceAV;
     ToxAV *BobAV = ((thread_data *) pd)->BobAV;
-    CallControl *AliceCC = ((thread_data *) pd)->AliceCC;
-    CallControl *BobCC = ((thread_data *) pd)->BobCC;
     uint32_t friend_number = ((thread_data *) pd)->friend_number;
 
-    memset(AliceCC, 0, sizeof(CallControl));
-    memset(BobCC, 0, sizeof(CallControl));
-
-    { /* Call */
-        Toxav_Err_Call rc;
-        toxav_call(AliceAV, friend_number, 48, 3000, &rc);
-
-        if (rc != TOXAV_ERR_CALL_OK) {
-            printf("toxav_call failed: %d\n", rc);
-            ck_assert(0);
-        }
-    }
-
-    do {
-        c_sleep(10);
-    } while (!BobCC->incoming);
-
-    { /* Answer */
-        Toxav_Err_Answer rc;
-        toxav_answer(BobAV, 0, 8, 500, &rc);
-
-        if (rc != TOXAV_ERR_ANSWER_OK) {
-            printf("toxav_answer failed: %d\n", rc);
-            ck_assert(0);
-        }
-    }
-
-    c_sleep(30);
-
     int16_t *PCM = (int16_t *)calloc(960, sizeof(int16_t));
     uint8_t *video_y = (uint8_t *)calloc(800 * 600, sizeof(uint8_t));
     uint8_t *video_u = (uint8_t *)calloc(800 * 600 / 4, sizeof(uint8_t));
@@ -152,17 +119,6 @@ static void *call_thread(void *pd)
         c_sleep(10);
     } while (time(nullptr) - start_time < 4);
 
-    { /* Hangup */
-        Toxav_Err_Call_Control rc;
-        toxav_call_control(AliceAV, friend_number, TOXAV_CALL_CONTROL_CANCEL, &rc);
-
-        if (rc != TOXAV_ERR_CALL_CONTROL_OK) {
-            printf("toxav_call_control failed: %d %p %p\n", rc, (void *)AliceAV, (void *)BobAV);
-        }
-    }
-
-    c_sleep(30);
-
     free(PCM);
     free(video_y);
     free(video_u);
@@ -174,6 +130,33 @@ static void *call_thread(void *pd)
     return nullptr;
 }
 
+typedef struct Time_Data {
+    pthread_mutex_t lock;
+    uint64_t clock;
+} Time_Data;
+
+static uint64_t get_state_clock_callback(void *user_data)
+{
+    Time_Data *time_data = (Time_Data *)user_data;
+    pthread_mutex_lock(&time_data->lock);
+    uint64_t clock = time_data->clock;
+    pthread_mutex_unlock(&time_data->lock);
+    return clock;
+}
+
+static void increment_clock(Time_Data *time_data, uint64_t count)
+{
+    pthread_mutex_lock(&time_data->lock);
+    time_data->clock += count;
+    pthread_mutex_unlock(&time_data->lock);
+}
+
+static void set_current_time_callback(Tox *tox, Time_Data *time_data)
+{
+    Mono_Time *mono_time = tox->mono_time;
+    mono_time_set_current_time_callback(mono_time, get_state_clock_callback, time_data);
+}
+
 static void test_av_three_calls(void)
 {
     uint32_t index[] = { 1, 2, 3, 4, 5 };
@@ -183,23 +166,32 @@ static void test_av_three_calls(void)
 
     CallControl AliceCC[3], BobsCC[3];
 
+    Time_Data time_data;
+    pthread_mutex_init(&time_data.lock, nullptr);
     {
         Tox_Err_New error;
 
         bootstrap = tox_new_log(nullptr, &error, &index[0]);
         ck_assert(error == TOX_ERR_NEW_OK);
 
+        time_data.clock = current_time_monotonic(bootstrap->mono_time);
+        set_current_time_callback(bootstrap, &time_data);
+
         Alice = tox_new_log(nullptr, &error, &index[1]);
         ck_assert(error == TOX_ERR_NEW_OK);
+        set_current_time_callback(Alice, &time_data);
 
         Bobs[0] = tox_new_log(nullptr, &error, &index[2]);
         ck_assert(error == TOX_ERR_NEW_OK);
+        set_current_time_callback(Bobs[0], &time_data);
 
         Bobs[1] = tox_new_log(nullptr, &error, &index[3]);
         ck_assert(error == TOX_ERR_NEW_OK);
+        set_current_time_callback(Bobs[1], &time_data);
 
         Bobs[2] = tox_new_log(nullptr, &error, &index[4]);
         ck_assert(error == TOX_ERR_NEW_OK);
+        set_current_time_callback(Bobs[2], &time_data);
     }
 
     printf("Created 5 instances of Tox\n");
@@ -222,9 +214,9 @@ static void test_av_three_calls(void)
     tox_bootstrap(Bobs[1], "localhost", dht_port, dht_key, nullptr);
     tox_bootstrap(Bobs[2], "localhost", dht_port, dht_key, nullptr);
 
-    ck_assert(tox_friend_add(Bobs[0], address, (const uint8_t *)"gentoo", 7, nullptr) != (uint32_t) ~0);
-    ck_assert(tox_friend_add(Bobs[1], address, (const uint8_t *)"gentoo", 7, nullptr) != (uint32_t) ~0);
-    ck_assert(tox_friend_add(Bobs[2], address, (const uint8_t *)"gentoo", 7, nullptr) != (uint32_t) ~0);
+    ck_assert(tox_friend_add(Bobs[0], address, (const uint8_t *)"gentoo", 7, nullptr) != (uint32_t) -1);
+    ck_assert(tox_friend_add(Bobs[1], address, (const uint8_t *)"gentoo", 7, nullptr) != (uint32_t) -1);
+    ck_assert(tox_friend_add(Bobs[2], address, (const uint8_t *)"gentoo", 7, nullptr) != (uint32_t) -1);
 
     uint8_t off = 1;
 
@@ -253,40 +245,35 @@ static void test_av_three_calls(void)
             break;
         }
 
-        c_sleep(20);
+        increment_clock(&time_data, 200);
+        c_sleep(5);
     }
 
     AliceAV = setup_av_instance(Alice, AliceCC);
-    BobsAV[0] = setup_av_instance(Bobs[0], BobsCC + 0);
-    BobsAV[1] = setup_av_instance(Bobs[1], BobsCC + 1);
-    BobsAV[2] = setup_av_instance(Bobs[2], BobsCC + 2);
+    BobsAV[0] = setup_av_instance(Bobs[0], &BobsCC[0]);
+    BobsAV[1] = setup_av_instance(Bobs[1], &BobsCC[1]);
+    BobsAV[2] = setup_av_instance(Bobs[2], &BobsCC[2]);
 
     printf("Created 4 instances of ToxAV\n");
     printf("All set after %lu seconds!\n", (unsigned long)(time(nullptr) - cur_time));
 
     thread_data tds[3];
-    tds[0].AliceAV = AliceAV;
-    tds[0].BobAV = BobsAV[0];
-    tds[0].AliceCC = AliceCC + 0;
-    tds[0].BobCC = BobsCC + 0;
-    tds[0].friend_number = 0;
 
-    tds[1].AliceAV = AliceAV;
-    tds[1].BobAV = BobsAV[1];
-    tds[1].AliceCC = AliceCC + 1;
-    tds[1].BobCC = BobsCC + 1;
-    tds[1].friend_number = 1;
-
-    tds[2].AliceAV = AliceAV;
-    tds[2].BobAV = BobsAV[2];
-    tds[2].AliceCC = AliceCC + 2;
-    tds[2].BobCC = BobsCC + 2;
-    tds[2].friend_number = 2;
+    for (size_t i = 0; i < 3; i++) {
+        tds[i].AliceAV = AliceAV;
+        tds[i].BobAV = BobsAV[i];
+        tds[i].AliceCC = &AliceCC[i];
+        tds[i].BobCC = &BobsCC[i];
+        tds[i].friend_number = i;
+        memset(tds[i].AliceCC, 0, sizeof(CallControl));
+        memset(tds[i].BobCC, 0, sizeof(CallControl));
+    }
 
     pthread_t tids[3];
-    (void) pthread_create(tids + 0, nullptr, call_thread, tds + 0);
-    (void) pthread_create(tids + 1, nullptr, call_thread, tds + 1);
-    (void) pthread_create(tids + 2, nullptr, call_thread, tds + 2);
+
+    for (size_t i = 0; i < 3; i++) {
+        (void) pthread_create(&tids[i], nullptr, call_thread, &tds[i]);
+    }
 
     time_t start_time = time(nullptr);
 
@@ -296,7 +283,68 @@ static void test_av_three_calls(void)
         tox_iterate(Bobs[0], nullptr);
         tox_iterate(Bobs[1], nullptr);
         tox_iterate(Bobs[2], nullptr);
-        c_sleep(20);
+
+        increment_clock(&time_data, 100);
+        c_sleep(5);
+    } while (time(nullptr) - start_time < 1);
+
+    /* Call */
+    for (size_t i = 0; i < 3; i++) {
+        Toxav_Err_Call rc;
+        toxav_call(AliceAV, tds[i].friend_number, 48, 3000, &rc);
+
+        if (rc != TOXAV_ERR_CALL_OK) {
+            printf("toxav_call failed: %d\n", rc);
+            ck_assert(0);
+        }
+    }
+
+
+    do {
+        tox_iterate(bootstrap, nullptr);
+        tox_iterate(Alice, nullptr);
+        tox_iterate(Bobs[0], nullptr);
+        tox_iterate(Bobs[1], nullptr);
+        tox_iterate(Bobs[2], nullptr);
+
+        for (size_t i = 0; i < 3; i++) {
+            if (BobsCC[i].incoming) {
+                /* Answer */
+                Toxav_Err_Answer rc;
+                toxav_answer(BobsAV[i], 0, 8, 500, &rc);
+
+                if (rc != TOXAV_ERR_ANSWER_OK) {
+                    printf("toxav_answer failed: %d\n", rc);
+                    ck_assert(0);
+                }
+
+                BobsCC[i].incoming = false;
+            }
+        }
+
+        increment_clock(&time_data, 100);
+        c_sleep(5);
+    } while (time(nullptr) - start_time < 3);
+
+    /* Hangup */
+    for (size_t i = 0; i < 3; i++) {
+        Toxav_Err_Call_Control rc;
+        toxav_call_control(AliceAV, i, TOXAV_CALL_CONTROL_CANCEL, &rc);
+
+        if (rc != TOXAV_ERR_CALL_CONTROL_OK) {
+            printf("toxav_call_control failed: %d %p %p\n", rc, (void *)AliceAV, (void *)&BobsAV[i]);
+        }
+    }
+
+    do {
+        tox_iterate(bootstrap, nullptr);
+        tox_iterate(Alice, nullptr);
+        tox_iterate(Bobs[0], nullptr);
+        tox_iterate(Bobs[1], nullptr);
+        tox_iterate(Bobs[2], nullptr);
+
+        increment_clock(&time_data, 100);
+        c_sleep(5);
     } while (time(nullptr) - start_time < 5);
 
     ck_assert(pthread_join(tids[0], &retval) == 0);
@@ -319,6 +367,8 @@ static void test_av_three_calls(void)
     tox_kill(Alice);
     tox_kill(bootstrap);
 
+    pthread_mutex_destroy(&time_data.lock);
+
     printf("\nTest successful!\n");
 }
 
