@@ -2,17 +2,15 @@
  * Copyright © 2016-2018 The TokTok team.
  * Copyright © 2013-2015 Tox project.
  */
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif /* HAVE_CONFIG_H */
-
 #include "audio.h"
 
+#include <assert.h>
 #include <stdlib.h>
 #include <string.h>
 
 #include "rtp.h"
 
+#include "../toxcore/ccompat.h"
 #include "../toxcore/logger.h"
 #include "../toxcore/mono_time.h"
 
@@ -21,20 +19,20 @@ static void jbuf_clear(struct JitterBuffer *q);
 static void jbuf_free(struct JitterBuffer *q);
 static int jbuf_write(const Logger *log, struct JitterBuffer *q, struct RTPMessage *m);
 static struct RTPMessage *jbuf_read(struct JitterBuffer *q, int32_t *success);
-static OpusEncoder *create_audio_encoder(const Logger *log, int32_t bit_rate, int32_t sampling_rate,
-        int32_t channel_count);
-static bool reconfigure_audio_encoder(const Logger *log, OpusEncoder **e, int32_t new_br, int32_t new_sr,
-                                      uint8_t new_ch, int32_t *old_br, int32_t *old_sr, int32_t *old_ch);
-static bool reconfigure_audio_decoder(ACSession *ac, int32_t sampling_rate, int8_t channels);
+static OpusEncoder *create_audio_encoder(const Logger *log, uint32_t bit_rate, uint32_t sampling_rate,
+        uint8_t channel_count);
+static bool reconfigure_audio_encoder(const Logger *log, OpusEncoder **e, uint32_t new_br, uint32_t new_sr,
+                                      uint8_t new_ch, uint32_t *old_br, uint32_t *old_sr, uint8_t *old_ch);
+static bool reconfigure_audio_decoder(ACSession *ac, uint32_t sampling_rate, uint8_t channels);
 
 
 
 ACSession *ac_new(Mono_Time *mono_time, const Logger *log, ToxAV *av, uint32_t friend_number,
                   toxav_audio_receive_frame_cb *cb, void *cb_data)
 {
-    ACSession *ac = (ACSession *)calloc(sizeof(ACSession), 1);
+    ACSession *ac = (ACSession *)calloc(1, sizeof(ACSession));
 
-    if (!ac) {
+    if (ac == nullptr) {
         LOGGER_WARNING(log, "Allocation failed! Application might misbehave!");
         return nullptr;
     }
@@ -103,7 +101,7 @@ BASE_CLEANUP:
 
 void ac_kill(ACSession *ac)
 {
-    if (!ac) {
+    if (ac == nullptr) {
         return;
     }
 
@@ -119,7 +117,7 @@ void ac_kill(ACSession *ac)
 
 void ac_iterate(ACSession *ac)
 {
-    if (!ac) {
+    if (ac == nullptr) {
         return;
     }
 
@@ -137,31 +135,16 @@ void ac_iterate(ACSession *ac)
     struct JitterBuffer *const j_buf = (struct JitterBuffer *)ac->j_buf;
 
     int rc = 0;
-    struct RTPMessage *msg = jbuf_read(j_buf, &rc);
 
-    for (; msg != nullptr || rc == 2; msg = jbuf_read(j_buf, &rc)) {
+    for (struct RTPMessage *msg = jbuf_read(j_buf, &rc); msg != nullptr || rc == 2; msg = jbuf_read(j_buf, &rc)) {
         pthread_mutex_unlock(ac->queue_mutex);
 
         if (rc == 2) {
             LOGGER_DEBUG(ac->log, "OPUS correction");
-            int fs = (ac->lp_sampling_rate * ac->lp_frame_duration) / 1000;
+            const int fs = (ac->lp_sampling_rate * ac->lp_frame_duration) / 1000;
             rc = opus_decode(ac->decoder, nullptr, 0, temp_audio_buffer, fs, 1);
         } else {
-            /* Get values from packet and decode. */
-            /* NOTE: This didn't work very well */
-#if 0
-            rc = convert_bw_to_sampling_rate(opus_packet_get_bandwidth(msg->data));
-
-            if (rc != -1) {
-                cs->last_packet_sampling_rate = rc;
-            } else {
-                LOGGER_WARNING(ac->log, "Failed to load packet values!");
-                rtp_free_msg(msg);
-                continue;
-            }
-
-#endif
-
+            assert(msg->len > 4);
 
             /* Pick up sampling rate from packet */
             memcpy(&ac->lp_sampling_rate, msg->data, 4);
@@ -169,12 +152,13 @@ void ac_iterate(ACSession *ac)
 
             ac->lp_channel_count = opus_packet_get_nb_channels(msg->data + 4);
 
-            /** NOTE: even though OPUS supports decoding mono frames with stereo decoder and vice versa,
-              * it didn't work quite well.
-              */
+            /* NOTE: even though OPUS supports decoding mono frames with stereo decoder and vice versa,
+             * it didn't work quite well.
+             */
             if (!reconfigure_audio_decoder(ac, ac->lp_sampling_rate, ac->lp_channel_count)) {
                 LOGGER_WARNING(ac->log, "Failed to reconfigure decoder!");
                 free(msg);
+                pthread_mutex_lock(ac->queue_mutex);
                 continue;
             }
 
@@ -193,7 +177,7 @@ void ac_iterate(ACSession *ac)
 
         if (rc < 0) {
             LOGGER_WARNING(ac->log, "Decoding error: %s", opus_strerror(rc));
-        } else if (ac->acb) {
+        } else if (ac->acb != nullptr) {
             ac->lp_frame_duration = (rc * 1000) / ac->lp_sampling_rate;
 
             ac->acb(ac->av, ac->friend_number, temp_audio_buffer, rc, ac->lp_channel_count,
@@ -212,11 +196,8 @@ void ac_iterate(ACSession *ac)
 
 int ac_queue_message(Mono_Time *mono_time, void *acp, struct RTPMessage *msg)
 {
-    if (!acp || !msg) {
-        if (msg) {
-            free(msg);
-        }
-
+    if (acp == nullptr || msg == nullptr) {
+        free(msg);
         return -1;
     }
 
@@ -235,7 +216,7 @@ int ac_queue_message(Mono_Time *mono_time, void *acp, struct RTPMessage *msg)
     }
 
     pthread_mutex_lock(ac->queue_mutex);
-    int rc = jbuf_write(ac->log, (struct JitterBuffer *)ac->j_buf, msg);
+    const int rc = jbuf_write(ac->log, (struct JitterBuffer *)ac->j_buf, msg);
     pthread_mutex_unlock(ac->queue_mutex);
 
     if (rc == -1) {
@@ -247,13 +228,14 @@ int ac_queue_message(Mono_Time *mono_time, void *acp, struct RTPMessage *msg)
     return 0;
 }
 
-int ac_reconfigure_encoder(ACSession *ac, int32_t bit_rate, int32_t sampling_rate, uint8_t channels)
+int ac_reconfigure_encoder(ACSession *ac, uint32_t bit_rate, uint32_t sampling_rate, uint8_t channels)
 {
-    if (!ac || !reconfigure_audio_encoder(ac->log, &ac->encoder, bit_rate,
-                                          sampling_rate, channels,
-                                          &ac->le_bit_rate,
-                                          &ac->le_sample_rate,
-                                          &ac->le_channel_count)) {
+    if (ac == nullptr || !reconfigure_audio_encoder(
+                ac->log, &ac->encoder, bit_rate,
+                sampling_rate, channels,
+                &ac->le_bit_rate,
+                &ac->le_sample_rate,
+                &ac->le_channel_count)) {
         return -1;
     }
 
@@ -278,15 +260,15 @@ static struct JitterBuffer *jbuf_new(uint32_t capacity)
         size *= 2;
     }
 
-    struct JitterBuffer *q = (struct JitterBuffer *)calloc(sizeof(struct JitterBuffer), 1);
+    struct JitterBuffer *q = (struct JitterBuffer *)calloc(1, sizeof(struct JitterBuffer));
 
-    if (!q) {
+    if (q == nullptr) {
         return nullptr;
     }
 
-    q->queue = (struct RTPMessage **)calloc(sizeof(struct RTPMessage *), size);
+    q->queue = (struct RTPMessage **)calloc(size, sizeof(struct RTPMessage *));
 
-    if (!q->queue) {
+    if (q->queue == nullptr) {
         free(q);
         return nullptr;
     }
@@ -297,16 +279,15 @@ static struct JitterBuffer *jbuf_new(uint32_t capacity)
 }
 static void jbuf_clear(struct JitterBuffer *q)
 {
-    for (; q->bottom != q->top; ++q->bottom) {
-        if (q->queue[q->bottom % q->size]) {
-            free(q->queue[q->bottom % q->size]);
-            q->queue[q->bottom % q->size] = nullptr;
-        }
+    while (q->bottom != q->top) {
+        free(q->queue[q->bottom % q->size]);
+        q->queue[q->bottom % q->size] = nullptr;
+        ++q->bottom;
     }
 }
 static void jbuf_free(struct JitterBuffer *q)
 {
-    if (!q) {
+    if (q == nullptr) {
         return;
     }
 
@@ -316,9 +297,9 @@ static void jbuf_free(struct JitterBuffer *q)
 }
 static int jbuf_write(const Logger *log, struct JitterBuffer *q, struct RTPMessage *m)
 {
-    uint16_t sequnum = m->header.sequnum;
+    const uint16_t sequnum = m->header.sequnum;
 
-    unsigned int num = sequnum % q->size;
+    const unsigned int num = sequnum % q->size;
 
     if ((uint32_t)(sequnum - q->bottom) > q->size) {
         LOGGER_DEBUG(log, "Clearing filled jitter buffer: %p", (void *)q);
@@ -330,7 +311,7 @@ static int jbuf_write(const Logger *log, struct JitterBuffer *q, struct RTPMessa
         return 0;
     }
 
-    if (q->queue[num]) {
+    if (q->queue[num] != nullptr) {
         return -1;
     }
 
@@ -349,9 +330,9 @@ static struct RTPMessage *jbuf_read(struct JitterBuffer *q, int32_t *success)
         return nullptr;
     }
 
-    unsigned int num = q->bottom % q->size;
+    const unsigned int num = q->bottom % q->size;
 
-    if (q->queue[num]) {
+    if (q->queue[num] != nullptr) {
         struct RTPMessage *ret = q->queue[num];
         q->queue[num] = nullptr;
         ++q->bottom;
@@ -368,8 +349,8 @@ static struct RTPMessage *jbuf_read(struct JitterBuffer *q, int32_t *success)
     *success = 0;
     return nullptr;
 }
-static OpusEncoder *create_audio_encoder(const Logger *log, int32_t bit_rate, int32_t sampling_rate,
-        int32_t channel_count)
+static OpusEncoder *create_audio_encoder(const Logger *log, uint32_t bit_rate, uint32_t sampling_rate,
+        uint8_t channel_count)
 {
     int status = OPUS_OK;
     /*
@@ -462,8 +443,8 @@ FAILURE:
     return nullptr;
 }
 
-static bool reconfigure_audio_encoder(const Logger *log, OpusEncoder **e, int32_t new_br, int32_t new_sr,
-                                      uint8_t new_ch, int32_t *old_br, int32_t *old_sr, int32_t *old_ch)
+static bool reconfigure_audio_encoder(const Logger *log, OpusEncoder **e, uint32_t new_br, uint32_t new_sr,
+                                      uint8_t new_ch, uint32_t *old_br, uint32_t *old_sr, uint8_t *old_ch)
 {
     /* Values are checked in toxav.c */
     if (*old_sr != new_sr || *old_ch != new_ch) {
@@ -479,7 +460,7 @@ static bool reconfigure_audio_encoder(const Logger *log, OpusEncoder **e, int32_
         return true; /* Nothing changed */
     }
 
-    int status = opus_encoder_ctl(*e, OPUS_SET_BITRATE(new_br));
+    const int status = opus_encoder_ctl(*e, OPUS_SET_BITRATE(new_br));
 
     if (status != OPUS_OK) {
         LOGGER_ERROR(log, "Error while setting encoder ctl: %s", opus_strerror(status));
@@ -494,7 +475,7 @@ static bool reconfigure_audio_encoder(const Logger *log, OpusEncoder **e, int32_
     return true;
 }
 
-static bool reconfigure_audio_decoder(ACSession *ac, int32_t sampling_rate, int8_t channels)
+static bool reconfigure_audio_decoder(ACSession *ac, uint32_t sampling_rate, uint8_t channels)
 {
     if (sampling_rate != ac->ld_sample_rate || channels != ac->ld_channel_count) {
         if (current_time_monotonic(ac->mono_time) - ac->ldrts < 500) {
