@@ -2,10 +2,6 @@
  * Copyright © 2016-2018 The TokTok team.
  * Copyright © 2013-2015 Tox project.
  */
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif /* HAVE_CONFIG_H */
-
 #include "bwcontroller.h"
 
 #include <assert.h>
@@ -15,6 +11,7 @@
 
 #include "ring_buffer.h"
 
+#include "../toxcore/ccompat.h"
 #include "../toxcore/logger.h"
 #include "../toxcore/mono_time.h"
 #include "../toxcore/util.h"
@@ -38,7 +35,7 @@ typedef struct BWCRcvPkt {
     RingBuffer *rb;
 } BWCRcvPkt;
 
-struct BWController_s {
+struct BWController {
     m_cb *mcb;
     void *mcb_user_data;
 
@@ -66,7 +63,7 @@ static void send_update(BWController *bwc);
 BWController *bwc_new(Messenger *m, Tox *tox, uint32_t friendnumber, m_cb *mcb, void *mcb_user_data,
                       Mono_Time *bwc_mono_time)
 {
-    BWController *retu = (BWController *)calloc(sizeof(struct BWController_s), 1);
+    BWController *retu = (BWController *)calloc(1, sizeof(BWController));
 
     if (retu == nullptr) {
         return nullptr;
@@ -78,7 +75,7 @@ BWController *bwc_new(Messenger *m, Tox *tox, uint32_t friendnumber, m_cb *mcb,
     retu->m = m;
     retu->friend_number = friendnumber;
     retu->bwc_mono_time = bwc_mono_time;
-    uint64_t now = current_time_monotonic(bwc_mono_time);
+    const uint64_t now = current_time_monotonic(bwc_mono_time);
     retu->cycle.last_sent_timestamp = now;
     retu->cycle.last_refresh_timestamp = now;
     retu->tox = tox;
@@ -98,7 +95,7 @@ BWController *bwc_new(Messenger *m, Tox *tox, uint32_t friendnumber, m_cb *mcb,
 
 void bwc_kill(BWController *bwc)
 {
-    if (!bwc) {
+    if (bwc == nullptr) {
         return;
     }
 
@@ -109,7 +106,7 @@ void bwc_kill(BWController *bwc)
 
 void bwc_add_lost(BWController *bwc, uint32_t bytes_lost)
 {
-    if (!bwc) {
+    if (bwc == nullptr) {
         return;
     }
 
@@ -122,7 +119,7 @@ void bwc_add_lost(BWController *bwc, uint32_t bytes_lost)
 
 void bwc_add_recv(BWController *bwc, uint32_t recv_bytes)
 {
-    if (!bwc || !recv_bytes) {
+    if (bwc == nullptr || recv_bytes == 0) {
         return;
     }
 
@@ -137,10 +134,10 @@ static void send_update(BWController *bwc)
             current_time_monotonic(bwc->bwc_mono_time) - bwc->cycle.last_sent_timestamp > BWC_SEND_INTERVAL_MS) {
         bwc->packet_loss_counted_cycles = 0;
 
-        if (bwc->cycle.lost) {
+        if (bwc->cycle.lost != 0) {
             LOGGER_DEBUG(bwc->m->log, "%p Sent update rcv: %u lost: %u percent: %f %%",
                          (void *)bwc, bwc->cycle.recv, bwc->cycle.lost,
-                         (((double) bwc->cycle.lost / (bwc->cycle.recv + bwc->cycle.lost)) * 100.0));
+                         ((double)bwc->cycle.lost / (bwc->cycle.recv + bwc->cycle.lost)) * 100.0);
             uint8_t bwc_packet[sizeof(struct BWCMessage) + 1];
             size_t offset = 0;
 
@@ -152,9 +149,11 @@ static void send_update(BWController *bwc)
             assert(offset == sizeof(bwc_packet));
 
             if (bwc_send_custom_lossy_packet(bwc->tox, bwc->friend_number, bwc_packet, sizeof(bwc_packet)) == -1) {
-                const char *netstrerror = net_new_strerror(net_error());
+                char *netstrerror = net_new_strerror(net_error());
+                char *stdstrerror = net_new_strerror(errno);
                 LOGGER_WARNING(bwc->m->log, "BWC send failed (len: %u)! std error: %s, net error %s",
-                               (unsigned)sizeof(bwc_packet), strerror(errno), netstrerror);
+                               (unsigned)sizeof(bwc_packet), stdstrerror, netstrerror);
+                net_kill_strerror(stdstrerror);
                 net_kill_strerror(netstrerror);
             }
         }
@@ -177,14 +176,14 @@ static int on_update(BWController *bwc, const struct BWCMessage *msg)
 
     bwc->cycle.last_recv_timestamp = current_time_monotonic(bwc->bwc_mono_time);
 
-    const uint32_t recv = msg->recv;
     const uint32_t lost = msg->lost;
 
-    if (lost && bwc->mcb) {
+    if (lost != 0 && bwc->mcb != nullptr) {
+        const uint32_t recv = msg->recv;
         LOGGER_DEBUG(bwc->m->log, "recved: %u lost: %u percentage: %f %%", recv, lost,
-                     (((double) lost / (recv + lost)) * 100.0));
+                     ((double)lost / (recv + lost)) * 100.0);
         bwc->mcb(bwc, bwc->friend_number,
-                 ((float) lost / (recv + lost)),
+                 (float)lost / (recv + lost),
                  bwc->mcb_user_data);
     }
 
