@@ -5,12 +5,12 @@
 #ifndef C_TOXCORE_TOXAV_GROUPAV_H
 #define C_TOXCORE_TOXAV_GROUPAV_H
 
+// Audio encoding/decoding
+#include <opus.h>
+
 #include "../toxcore/group.h"
 #include "../toxcore/tox.h"
 
-/* Audio encoding/decoding */
-#include <opus.h>
-
 #define GROUP_AUDIO_PACKET_ID 192
 
 // TODO(iphydf): Use this better typed one instead of the void-pointer one below.
@@ -19,48 +19,46 @@
 typedef void audio_data_cb(void *tox, uint32_t groupnumber, uint32_t peernumber, const int16_t *pcm,
                            uint32_t samples, uint8_t channels, uint32_t sample_rate, void *userdata);
 
-/* Create and connect to a new toxav group.
+/** @brief Create and connect to a new toxav group.
  *
- * return group number on success.
- * return -1 on failure.
+ * @return group number on success.
+ * @retval -1 on failure.
  */
 int add_av_groupchat(const Logger *log, Tox *tox, Group_Chats *g_c, audio_data_cb *audio_callback, void *userdata);
 
-/* Join a AV group (you need to have been invited first.)
+/** @brief Join a AV group (you need to have been invited first).
  *
- * returns group number on success
- * returns -1 on failure.
+ * @return group number on success
+ * @retval -1 on failure.
  */
 int join_av_groupchat(const Logger *log, Tox *tox, Group_Chats *g_c, uint32_t friendnumber, const uint8_t *data,
-                      uint16_t length,
-                      audio_data_cb *audio_callback, void *userdata);
+                      uint16_t length, audio_data_cb *audio_callback, void *userdata);
 
 
-/* Send audio to the group chat.
+/** @brief Send audio to the group chat.
  *
- * return 0 on success.
- * return -1 on failure.
+ * @retval 0 on success.
+ * @retval -1 on failure.
  */
 int group_send_audio(Group_Chats *g_c, uint32_t groupnumber, const int16_t *pcm, unsigned int samples, uint8_t channels,
                      uint32_t sample_rate);
 
-/* Enable A/V in a groupchat.
+/** @brief Enable A/V in a groupchat.
  *
- * return 0 on success.
- * return -1 on failure.
+ * @retval 0 on success.
+ * @retval -1 on failure.
  */
 int groupchat_enable_av(const Logger *log, Tox *tox, Group_Chats *g_c, uint32_t groupnumber,
                         audio_data_cb *audio_callback, void *userdata);
 
-/* Disable A/V in a groupchat.
+/** @brief Disable A/V in a groupchat.
  *
- * return 0 on success.
- * return -1 on failure.
+ * @retval 0 on success.
+ * @retval -1 on failure.
  */
-int groupchat_disable_av(Group_Chats *g_c, uint32_t groupnumber);
+int groupchat_disable_av(const Group_Chats *g_c, uint32_t groupnumber);
 
-/* Return whether A/V is enabled in the groupchat.
- */
-bool groupchat_av_enabled(Group_Chats *g_c, uint32_t groupnumber);
+/** Return whether A/V is enabled in the groupchat. */
+bool groupchat_av_enabled(const Group_Chats *g_c, uint32_t groupnumber);
 
 #endif // C_TOXCORE_TOXAV_GROUPAV_H
