@@ -2,20 +2,17 @@
  * Copyright © 2016-2018 The TokTok team.
  * Copyright © 2013-2015 Tox project.
  */
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif /* HAVE_CONFIG_H */
-
 #include "msi.h"
 
-#include "../toxcore/logger.h"
-#include "../toxcore/util.h"
-
 #include <assert.h>
 #include <stdbool.h>
 #include <stdlib.h>
 #include <string.h>
 
+#include "../toxcore/ccompat.h"
+#include "../toxcore/logger.h"
+#include "../toxcore/util.h"
+
 #define MSI_MAXMSG_SIZE 256
 
 /**
@@ -67,7 +64,7 @@ static uint8_t *msg_parse_header_out(MSIHeaderID id, uint8_t *dest, const void *
                                      uint16_t *length);
 static int send_message(Messenger *m, uint32_t friend_number, const MSIMessage *msg);
 static int send_error(Messenger *m, uint32_t friend_number, MSIError error);
-static int invoke_callback(MSICall *call, MSICallbackID cb);
+static bool invoke_callback(MSICall *call, MSICallbackID cb);
 static MSICall *get_call(MSISession *session, uint32_t friend_number);
 static MSICall *new_call(MSISession *session, uint32_t friend_number);
 static void kill_call(MSICall *call);
@@ -78,26 +75,42 @@ static void handle_pop(MSICall *call, const MSIMessage *msg);
 static void handle_msi_packet(Messenger *m, uint32_t friend_number, const uint8_t *data, uint16_t length, void *object);
 
 
-/**
+/*
  * Public functions
  */
-void msi_register_callback(MSISession *session, msi_action_cb *callback, MSICallbackID id)
-{
-    if (!session) {
-        return;
-    }
 
-    pthread_mutex_lock(session->mutex);
-    session->callbacks[id] = callback;
-    pthread_mutex_unlock(session->mutex);
+void msi_callback_invite(MSISession *session, msi_action_cb *callback)
+{
+    session->invite_callback = callback;
 }
+void msi_callback_start(MSISession *session, msi_action_cb *callback)
+{
+    session->start_callback = callback;
+}
+void msi_callback_end(MSISession *session, msi_action_cb *callback)
+{
+    session->end_callback = callback;
+}
+void msi_callback_error(MSISession *session, msi_action_cb *callback)
+{
+    session->error_callback = callback;
+}
+void msi_callback_peertimeout(MSISession *session, msi_action_cb *callback)
+{
+    session->peertimeout_callback = callback;
+}
+void msi_callback_capabilities(MSISession *session, msi_action_cb *callback)
+{
+    session->capabilities_callback = callback;
+}
+
 MSISession *msi_new(Messenger *m)
 {
     if (m == nullptr) {
         return nullptr;
     }
 
-    MSISession *retu = (MSISession *)calloc(sizeof(MSISession), 1);
+    MSISession *retu = (MSISession *)calloc(1, sizeof(MSISession));
 
     if (retu == nullptr) {
         LOGGER_ERROR(m->log, "Allocation failed! Program might misbehave!");
@@ -134,13 +147,13 @@ int msi_kill(MSISession *session, const Logger *log)
         return -1;
     }
 
-    if (session->calls) {
+    if (session->calls != nullptr) {
         MSIMessage msg;
         msg_init(&msg, REQU_POP);
 
         MSICall *it = get_call(session, session->calls_head);
 
-        while (it) {
+        while (it != nullptr) {
             send_message(session->messenger, it->friend_number, &msg);
             MSICall *temp_it = it;
             it = it->next;
@@ -157,7 +170,7 @@ int msi_kill(MSISession *session, const Logger *log)
 }
 int msi_invite(MSISession *session, MSICall **call, uint32_t friend_number, uint8_t capabilities)
 {
-    if (!session) {
+    if (session == nullptr) {
         return -1;
     }
 
@@ -201,7 +214,7 @@ int msi_invite(MSISession *session, MSICall **call, uint32_t friend_number, uint
 }
 int msi_hangup(MSICall *call)
 {
-    if (!call || !call->session) {
+    if (call == nullptr || call->session == nullptr) {
         return -1;
     }
 
@@ -232,7 +245,7 @@ int msi_hangup(MSICall *call)
 }
 int msi_answer(MSICall *call, uint8_t capabilities)
 {
-    if (!call || !call->session) {
+    if (call == nullptr || call->session == nullptr) {
         return -1;
     }
 
@@ -271,7 +284,7 @@ int msi_answer(MSICall *call, uint8_t capabilities)
 }
 int msi_change_capabilities(MSICall *call, uint8_t capabilities)
 {
-    if (!call || !call->session) {
+    if (call == nullptr || call->session == nullptr) {
         return -1;
     }
 
@@ -315,35 +328,43 @@ static void msg_init(MSIMessage *dest, MSIRequest request)
     dest->request.exists = true;
     dest->request.value = request;
 }
+
+static bool check_size(const Logger *log, const uint8_t *bytes, int *constraint, uint8_t size)
+{
+    *constraint -= 2 + size;
+
+    if (*constraint < 1) {
+        LOGGER_ERROR(log, "Read over length!");
+        return false;
+    }
+
+    if (bytes[1] != size) {
+        LOGGER_ERROR(log, "Invalid data size!");
+        return false;
+    }
+
+    return true;
+}
+
+/** Assumes size == 1 */
+static bool check_enum_high(const Logger *log, const uint8_t *bytes, uint8_t enum_high)
+{
+    if (bytes[2] > enum_high) {
+        LOGGER_ERROR(log, "Failed enum high limit!");
+        return false;
+    }
+
+    return true;
+}
+
+
 static int msg_parse_in(const Logger *log, MSIMessage *dest, const uint8_t *data, uint16_t length)
 {
     /* Parse raw data received from socket into MSIMessage struct */
 
-#define CHECK_SIZE(bytes, constraint, size)          \
-    do {                                             \
-        constraint -= 2 + size;                      \
-        if (constraint < 1) {                        \
-            LOGGER_ERROR(log, "Read over length!");  \
-            return -1;                               \
-        }                                            \
-        if (bytes[1] != size) {                      \
-            LOGGER_ERROR(log, "Invalid data size!"); \
-            return -1;                               \
-        }                                            \
-    } while (0)
+    assert(dest != nullptr);
 
-    /* Assumes size == 1 */
-#define CHECK_ENUM_HIGH(bytes, enum_high)                 \
-    do {                                                  \
-        if (bytes[2] > enum_high) {                       \
-            LOGGER_ERROR(log, "Failed enum high limit!"); \
-            return -1;                                    \
-        }                                                 \
-    } while (0)
-
-    assert(dest);
-
-    if (length == 0 || data[length - 1]) { /* End byte must have value 0 */
+    if (length == 0 || data[length - 1] != 0) { /* End byte must have value 0 */
         LOGGER_ERROR(log, "Invalid end byte");
         return -1;
     }
@@ -353,54 +374,64 @@ static int msg_parse_in(const Logger *log, MSIMessage *dest, const uint8_t *data
     const uint8_t *it = data;
     int size_constraint = length;
 
-    while (*it) {/* until end byte is hit */
+    while (*it != 0) {/* until end byte is hit */
         switch (*it) {
-            case ID_REQUEST:
-                CHECK_SIZE(it, size_constraint, 1);
-                CHECK_ENUM_HIGH(it, REQU_POP);
+            case ID_REQUEST: {
+                if (!check_size(log, it, &size_constraint, 1) ||
+                        !check_enum_high(log, it, REQU_POP)) {
+                    return -1;
+                }
+
                 dest->request.value = (MSIRequest)it[2];
                 dest->request.exists = true;
                 it += 3;
                 break;
+            }
+
+            case ID_ERROR: {
+                if (!check_size(log, it, &size_constraint, 1) ||
+                        !check_enum_high(log, it, MSI_E_UNDISCLOSED)) {
+                    return -1;
+                }
 
-            case ID_ERROR:
-                CHECK_SIZE(it, size_constraint, 1);
-                CHECK_ENUM_HIGH(it, MSI_E_UNDISCLOSED);
                 dest->error.value = (MSIError)it[2];
                 dest->error.exists = true;
                 it += 3;
                 break;
+            }
+
+            case ID_CAPABILITIES: {
+                if (!check_size(log, it, &size_constraint, 1)) {
+                    return -1;
+                }
 
-            case ID_CAPABILITIES:
-                CHECK_SIZE(it, size_constraint, 1);
                 dest->capabilities.value = it[2];
                 dest->capabilities.exists = true;
                 it += 3;
                 break;
+            }
 
-            default:
+            default: {
                 LOGGER_ERROR(log, "Invalid id byte");
                 return -1;
+            }
         }
     }
 
-    if (dest->request.exists == false) {
+    if (!dest->request.exists) {
         LOGGER_ERROR(log, "Invalid request field!");
         return -1;
     }
 
-#undef CHECK_ENUM_HIGH
-#undef CHECK_SIZE
-
     return 0;
 }
 static uint8_t *msg_parse_header_out(MSIHeaderID id, uint8_t *dest, const void *value, uint8_t value_len,
                                      uint16_t *length)
 {
     /* Parse a single header for sending */
-    assert(dest);
-    assert(value);
-    assert(value_len);
+    assert(dest != nullptr);
+    assert(value != nullptr);
+    assert(value_len != 0);
 
     *dest = id;
     ++dest;
@@ -409,14 +440,14 @@ static uint8_t *msg_parse_header_out(MSIHeaderID id, uint8_t *dest, const void *
 
     memcpy(dest, value, value_len);
 
-    *length += (2 + value_len);
+    *length += 2 + value_len;
 
     return dest + value_len; /* Set to next position ready to be written */
 }
 static int send_message(Messenger *m, uint32_t friend_number, const MSIMessage *msg)
 {
     /* Parse and send message */
-    assert(m);
+    assert(m != nullptr);
 
     uint8_t parsed [MSI_MAXMSG_SIZE];
 
@@ -461,7 +492,7 @@ static int send_message(Messenger *m, uint32_t friend_number, const MSIMessage *
 static int send_error(Messenger *m, uint32_t friend_number, MSIError error)
 {
     /* Send error message */
-    assert(m);
+    assert(m != nullptr);
 
     LOGGER_DEBUG(m->log, "Sending error: %d to friend: %d", error, friend_number);
 
@@ -474,36 +505,57 @@ static int send_error(Messenger *m, uint32_t friend_number, MSIError error)
     send_message(m, friend_number, &msg);
     return 0;
 }
-static int invoke_callback(MSICall *call, MSICallbackID cb)
+static int invoke_callback_inner(MSICall *call, MSICallbackID id)
 {
-    assert(call);
+    MSISession *session = call->session;
+    LOGGER_DEBUG(session->messenger->log, "invoking callback function: %d", id);
 
-    if (call->session->callbacks[cb]) {
-        LOGGER_DEBUG(call->session->messenger->log, "Invoking callback function: %d", cb);
+    switch (id) {
+        case MSI_ON_INVITE:
+            return session->invite_callback(session->av, call);
 
-        if (call->session->callbacks[cb](call->session->av, call) != 0) {
-            LOGGER_WARNING(call->session->messenger->log,
-                           "Callback state handling failed, sending error");
-            goto FAILURE;
+        case MSI_ON_START:
+            return session->start_callback(session->av, call);
+
+        case MSI_ON_END:
+            return session->end_callback(session->av, call);
+
+        case MSI_ON_ERROR:
+            return session->error_callback(session->av, call);
+
+        case MSI_ON_PEERTIMEOUT:
+            return session->peertimeout_callback(session->av, call);
+
+        case MSI_ON_CAPABILITIES:
+            return session->capabilities_callback(session->av, call);
+    }
+
+    LOGGER_FATAL(session->messenger->log, "invalid callback id: %d", id);
+    return -1;
+}
+static bool invoke_callback(MSICall *call, MSICallbackID cb)
+{
+    assert(call != nullptr);
+
+    if (invoke_callback_inner(call, cb) != 0) {
+        LOGGER_WARNING(call->session->messenger->log,
+                       "Callback state handling failed, sending error");
+
+        /* If no callback present or error happened while handling,
+         * an error message will be sent to friend
+         */
+        if (call->error == MSI_E_NONE) {
+            call->error = MSI_E_HANDLE;
         }
 
-        return 0;
+        return false;
     }
 
-FAILURE:
-    /* If no callback present or error happened while handling,
-     * an error message will be sent to friend
-     */
-
-    if (call->error == MSI_E_NONE) {
-        call->error = MSI_E_HANDLE;
-    }
-
-    return -1;
+    return true;
 }
 static MSICall *get_call(MSISession *session, uint32_t friend_number)
 {
-    assert(session);
+    assert(session != nullptr);
 
     if (session->calls == nullptr || session->calls_tail < friend_number) {
         return nullptr;
@@ -513,9 +565,9 @@ static MSICall *get_call(MSISession *session, uint32_t friend_number)
 }
 static MSICall *new_call(MSISession *session, uint32_t friend_number)
 {
-    assert(session);
+    assert(session != nullptr);
 
-    MSICall *rc = (MSICall *)calloc(sizeof(MSICall), 1);
+    MSICall *rc = (MSICall *)calloc(1, sizeof(MSICall));
 
     if (rc == nullptr) {
         return nullptr;
@@ -525,7 +577,7 @@ static MSICall *new_call(MSISession *session, uint32_t friend_number)
     rc->friend_number = friend_number;
 
     if (session->calls == nullptr) { /* Creating */
-        session->calls = (MSICall **)calloc(sizeof(MSICall *), friend_number + 1);
+        session->calls = (MSICall **)calloc(friend_number + 1, sizeof(MSICall *));
 
         if (session->calls == nullptr) {
             free(rc);
@@ -545,9 +597,7 @@ static MSICall *new_call(MSISession *session, uint32_t friend_number)
         session->calls = tmp;
 
         /* Set fields in between to null */
-        uint32_t i = session->calls_tail + 1;
-
-        for (; i < friend_number; ++i) {
+        for (uint32_t i = session->calls_tail + 1; i < friend_number; ++i) {
             session->calls[i] = nullptr;
         }
 
@@ -578,17 +628,17 @@ static void kill_call(MSICall *call)
     MSICall *prev = call->prev;
     MSICall *next = call->next;
 
-    if (prev) {
+    if (prev != nullptr) {
         prev->next = next;
-    } else if (next) {
+    } else if (next != nullptr) {
         session->calls_head = next->friend_number;
     } else {
         goto CLEAR_CONTAINER;
     }
 
-    if (next) {
+    if (next != nullptr) {
         next->prev = prev;
-    } else if (prev) {
+    } else if (prev != nullptr) {
         session->calls_tail = prev->friend_number;
     } else {
         goto CLEAR_CONTAINER;
@@ -607,40 +657,36 @@ CLEAR_CONTAINER:
 }
 static void on_peer_status(Messenger *m, uint32_t friend_number, uint8_t status, void *data)
 {
-    MSISession *session = (MSISession *)data;
-
-    switch (status) {
-        case 0: { /* Friend is now offline */
-            LOGGER_DEBUG(m->log, "Friend %d is now offline", friend_number);
-
-            pthread_mutex_lock(session->mutex);
-            MSICall *call = get_call(session, friend_number);
-
-            if (call == nullptr) {
-                pthread_mutex_unlock(session->mutex);
-                return;
-            }
-
-            invoke_callback(call, MSI_ON_PEERTIMEOUT); /* Failure is ignored */
-            kill_call(call);
-            pthread_mutex_unlock(session->mutex);
-        }
-        break;
-
-        default:
-            break;
+    if (status != 0) {
+        // Friend is online.
+        return;
     }
+
+    MSISession *session = (MSISession *)data;
+    LOGGER_DEBUG(m->log, "Friend %d is now offline", friend_number);
+
+    pthread_mutex_lock(session->mutex);
+    MSICall *call = get_call(session, friend_number);
+
+    if (call == nullptr) {
+        pthread_mutex_unlock(session->mutex);
+        return;
+    }
+
+    invoke_callback(call, MSI_ON_PEERTIMEOUT); /* Failure is ignored */
+    kill_call(call);
+    pthread_mutex_unlock(session->mutex);
 }
-static void handle_init(MSICall *call, const MSIMessage *msg)
+static bool try_handle_init(MSICall *call, const MSIMessage *msg)
 {
-    assert(call);
+    assert(call != nullptr);
     LOGGER_DEBUG(call->session->messenger->log,
                  "Session: %p Handling 'init' friend: %d", (void *)call->session, call->friend_number);
 
     if (!msg->capabilities.exists) {
         LOGGER_WARNING(call->session->messenger->log, "Session: %p Invalid capabilities on 'init'", (void *)call->session);
         call->error = MSI_E_INVALID_MESSAGE;
-        goto FAILURE;
+        return false;
     }
 
     switch (call->state) {
@@ -649,11 +695,12 @@ static void handle_init(MSICall *call, const MSIMessage *msg)
             call->peer_capabilities = msg->capabilities.value;
             call->state = MSI_CALL_REQUESTED;
 
-            if (invoke_callback(call, MSI_ON_INVITE) == -1) {
-                goto FAILURE;
+            if (!invoke_callback(call, MSI_ON_INVITE)) {
+                return false;
             }
+
+            break;
         }
-        break;
 
         case MSI_CALL_ACTIVE: {
             /* If peer sent init while the call is already
@@ -676,25 +723,33 @@ static void handle_init(MSICall *call, const MSIMessage *msg)
             /* If peer changed capabilities during re-call they will
              * be handled accordingly during the next step
              */
+            break;
         }
-        break;
 
         case MSI_CALL_REQUESTED: // fall-through
         case MSI_CALL_REQUESTING: {
             LOGGER_WARNING(call->session->messenger->log, "Session: %p Invalid state on 'init'", (void *)call->session);
             call->error = MSI_E_INVALID_STATE;
-            goto FAILURE;
+            return false;
         }
     }
 
-    return;
-FAILURE:
-    send_error(call->session->messenger, call->friend_number, call->error);
-    kill_call(call);
+    return true;
+}
+static void handle_init(MSICall *call, const MSIMessage *msg)
+{
+    assert(call != nullptr);
+    LOGGER_DEBUG(call->session->messenger->log,
+                 "Session: %p Handling 'init' friend: %d", (void *)call->session, call->friend_number);
+
+    if (!try_handle_init(call, msg)) {
+        send_error(call->session->messenger, call->friend_number, call->error);
+        kill_call(call);
+    }
 }
 static void handle_push(MSICall *call, const MSIMessage *msg)
 {
-    assert(call);
+    assert(call != nullptr);
 
     LOGGER_DEBUG(call->session->messenger->log, "Session: %p Handling 'push' friend: %d", (void *)call->session,
                  call->friend_number);
@@ -707,18 +762,19 @@ static void handle_push(MSICall *call, const MSIMessage *msg)
 
     switch (call->state) {
         case MSI_CALL_ACTIVE: {
-            /* Only act if capabilities changed */
             if (call->peer_capabilities != msg->capabilities.value) {
+                /* Only act if capabilities changed */
                 LOGGER_INFO(call->session->messenger->log, "Friend is changing capabilities to: %u", msg->capabilities.value);
 
                 call->peer_capabilities = msg->capabilities.value;
 
-                if (invoke_callback(call, MSI_ON_CAPABILITIES) == -1) {
+                if (!invoke_callback(call, MSI_ON_CAPABILITIES)) {
                     goto FAILURE;
                 }
             }
+
+            break;
         }
-        break;
 
         case MSI_CALL_REQUESTING: {
             LOGGER_INFO(call->session->messenger->log, "Friend answered our call");
@@ -727,18 +783,19 @@ static void handle_push(MSICall *call, const MSIMessage *msg)
             call->peer_capabilities = msg->capabilities.value;
             call->state = MSI_CALL_ACTIVE;
 
-            if (invoke_callback(call, MSI_ON_START) == -1) {
+            if (!invoke_callback(call, MSI_ON_START)) {
                 goto FAILURE;
             }
-        }
-        break;
 
-        /* Pushes during initialization state are ignored */
+            break;
+        }
+
         case MSI_CALL_INACTIVE: // fall-through
         case MSI_CALL_REQUESTED: {
+            /* Pushes during initialization state are ignored */
             LOGGER_WARNING(call->session->messenger->log, "Ignoring invalid push");
+            break;
         }
-        break;
     }
 
     return;
@@ -749,7 +806,7 @@ FAILURE:
 }
 static void handle_pop(MSICall *call, const MSIMessage *msg)
 {
-    assert(call);
+    assert(call != nullptr);
 
     LOGGER_DEBUG(call->session->messenger->log, "Session: %p Handling 'pop', friend id: %d", (void *)call->session,
                  call->friend_number);
@@ -763,30 +820,29 @@ static void handle_pop(MSICall *call, const MSIMessage *msg)
     } else {
         switch (call->state) {
             case MSI_CALL_INACTIVE: {
-                LOGGER_ERROR(call->session->messenger->log, "Handling what should be impossible case");
-                abort();
+                LOGGER_FATAL(call->session->messenger->log, "Handling what should be impossible case");
             }
 
             case MSI_CALL_ACTIVE: {
                 /* Hangup */
                 LOGGER_INFO(call->session->messenger->log, "Friend hung up on us");
                 invoke_callback(call, MSI_ON_END);
+                break;
             }
-            break;
 
             case MSI_CALL_REQUESTING: {
                 /* Reject */
                 LOGGER_INFO(call->session->messenger->log, "Friend rejected our call");
                 invoke_callback(call, MSI_ON_END);
+                break;
             }
-            break;
 
             case MSI_CALL_REQUESTED: {
                 /* Cancel */
                 LOGGER_INFO(call->session->messenger->log, "Friend canceled call invite");
                 invoke_callback(call, MSI_ON_END);
+                break;
             }
-            break;
         }
     }
 
@@ -827,17 +883,20 @@ static void handle_msi_packet(Messenger *m, uint32_t friend_number, const uint8_
     }
 
     switch (msg.request.value) {
-        case REQU_INIT:
+        case REQU_INIT: {
             handle_init(call, &msg);
             break;
+        }
 
-        case REQU_PUSH:
+        case REQU_PUSH: {
             handle_push(call, &msg);
             break;
+        }
 
-        case REQU_POP:
+        case REQU_POP: {
             handle_pop(call, &msg); /* always kills the call */
             break;
+        }
     }
 
     pthread_mutex_unlock(session->mutex);
