@@ -5,10 +5,10 @@
  */
 #include "ring_buffer.h"
 
-#include "../toxcore/ccompat.h"
-
 #include <stdlib.h>
 
+#include "../toxcore/ccompat.h"
+
 struct RingBuffer {
     uint16_t size; /* Max size */
     uint16_t start;
@@ -26,9 +26,9 @@ bool rb_empty(const RingBuffer *b)
     return b->end == b->start;
 }
 
-/*
- * returns: NULL on success
- *          input value "p" on FAILURE -> caller can free on failed rb_write
+/**
+ * @retval NULL on success
+ * @return input value "p" on failure, so caller can free on failed rb_write
  */
 void *rb_write(RingBuffer *b, void *p)
 {
@@ -66,16 +66,16 @@ bool rb_read(RingBuffer *b, void **p)
 
 RingBuffer *rb_new(int size)
 {
-    RingBuffer *buf = (RingBuffer *)calloc(sizeof(RingBuffer), 1);
+    RingBuffer *buf = (RingBuffer *)calloc(1, sizeof(RingBuffer));
 
-    if (!buf) {
+    if (buf == nullptr) {
         return nullptr;
     }
 
     buf->size = size + 1; /* include empty elem */
     buf->data = (void **)calloc(buf->size, sizeof(void *));
 
-    if (!buf->data) {
+    if (buf->data == nullptr) {
         free(buf);
         return nullptr;
     }
@@ -85,7 +85,7 @@ RingBuffer *rb_new(int size)
 
 void rb_kill(RingBuffer *b)
 {
-    if (b) {
+    if (b != nullptr) {
         free(b->data);
         free(b);
     }
@@ -105,9 +105,9 @@ uint16_t rb_size(const RingBuffer *b)
 
 uint16_t rb_data(const RingBuffer *b, void **dest)
 {
-    uint16_t i = 0;
+    uint16_t i;
 
-    for (; i < rb_size(b); ++i) {
+    for (i = 0; i < rb_size(b); ++i) {
         dest[i] = b->data[(b->start + i) % b->size];
     }
 
