@@ -2,10 +2,6 @@
  * Copyright © 2016-2018 The TokTok team.
  * Copyright © 2013-2015 Tox project.
  */
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif /* HAVE_CONFIG_H */
-
 #include "rtp.h"
 
 #include <assert.h>
@@ -16,6 +12,7 @@
 #include "bwcontroller.h"
 
 #include "../toxcore/Messenger.h"
+#include "../toxcore/ccompat.h"
 #include "../toxcore/logger.h"
 #include "../toxcore/mono_time.h"
 #include "../toxcore/util.h"
@@ -27,7 +24,7 @@
 #define VIDEO_KEEP_KEYFRAME_IN_BUFFER_FOR_MS 15
 
 
-/*
+/**
  * return -1 on failure, 0 on success
  *
  */
@@ -196,7 +193,7 @@ static int8_t get_slot(const Logger *log, struct RTPWorkBufferList *wkbl, bool i
  *
  * If there are no frames ready, we return NULL. If this function returns
  * non-NULL, it transfers ownership of the message to the caller, i.e. the
- * caller is responsible for storing it elsewhere or calling free().
+ * caller is responsible for storing it elsewhere or calling `free()`.
  */
 static struct RTPMessage *process_frame(const Logger *log, struct RTPWorkBufferList *wkbl, uint8_t slot_id)
 {
@@ -222,7 +219,7 @@ static struct RTPMessage *process_frame(const Logger *log, struct RTPWorkBufferL
     struct RTPMessage *const m_new = slot->buf;
     slot->buf = nullptr;
 
-    assert(wkbl->next_free_entry >= 1);
+    assert(wkbl->next_free_entry >= 1 && wkbl->next_free_entry <= USED_RTP_WORKBUFFER_COUNT);
 
     if (slot_id != wkbl->next_free_entry - 1) {
         // The slot is not the last slot, so we created a gap. We move all the
@@ -263,7 +260,7 @@ static bool fill_data_into_slot(const Logger *log, struct RTPWorkBufferList *wkb
     struct RTPWorkBuffer *const slot = &wkbl->work_buffer[slot_id];
 
     assert(header != nullptr);
-    assert(is_keyframe == (bool)(header->flags & RTP_KEY_FRAME));
+    assert(is_keyframe == (bool)((header->flags & RTP_KEY_FRAME) != 0));
 
     if (slot->received_len == 0) {
         assert(slot->buf == nullptr);
@@ -318,13 +315,13 @@ static void update_bwc_values(const Logger *log, RTPSession *session, const stru
     if (session->first_packets_counter < DISMISS_FIRST_LOST_VIDEO_PACKET_COUNT) {
         ++session->first_packets_counter;
     } else {
-        uint32_t data_length_full = msg->header.data_length_full; // without header
-        uint32_t received_length_full = msg->header.received_length_full; // without header
+        const uint32_t data_length_full = msg->header.data_length_full; // without header
+        const uint32_t received_length_full = msg->header.received_length_full; // without header
         bwc_add_recv(session->bwc, data_length_full);
 
         if (received_length_full < data_length_full) {
             LOGGER_DEBUG(log, "BWC: full length=%u received length=%d", data_length_full, received_length_full);
-            bwc_add_lost(session->bwc, (data_length_full - received_length_full));
+            bwc_add_lost(session->bwc, data_length_full - received_length_full);
         }
     }
 }
@@ -348,7 +345,8 @@ static void update_bwc_values(const Logger *log, RTPSession *session, const stru
  *   this is the actual payload length.
  * @param log A logger.
  *
- * @return -1 on error, 0 on success.
+ * @retval -1 on error.
+ * @retval 0 on success.
  */
 static int handle_video_packet(RTPSession *session, const struct RTPHeader *header,
                                const uint8_t *incoming_data, uint16_t incoming_data_length, const Logger *log)
@@ -427,7 +425,7 @@ static int handle_video_packet(RTPSession *session, const struct RTPHeader *head
 
     struct RTPMessage *m_new = process_frame(log, session->work_buffer_list, slot_id);
 
-    if (m_new) {
+    if (m_new != nullptr) {
         LOGGER_DEBUG(log, "-- handle_video_packet -- CALLBACK-003a b0=%d b1=%d", (int)m_new->data[0], (int)m_new->data[1]);
         update_bwc_values(log, session, m_new);
         session->mcb(session->m->mono_time, session->cs, m_new);
@@ -439,13 +437,14 @@ static int handle_video_packet(RTPSession *session, const struct RTPHeader *head
 }
 
 /**
- * @return -1 on error, 0 on success.
+ * @retval -1 on error.
+ * @retval 0 on success.
  */
 static int handle_rtp_packet(Messenger *m, uint32_t friendnumber, const uint8_t *data, uint16_t length, void *object)
 {
     RTPSession *session = (RTPSession *)object;
 
-    if (!session || length < RTP_HEADER_SIZE + 1) {
+    if (session == nullptr || length < RTP_HEADER_SIZE + 1) {
         LOGGER_WARNING(m->log, "No session or invalid length of received buffer!");
         return -1;
     }
@@ -471,7 +470,7 @@ static int handle_rtp_packet(Messenger *m, uint32_t friendnumber, const uint8_t
         return -1;
     }
 
-    if (header.flags & RTP_LARGE_FRAME && header.offset_full >= header.data_length_full) {
+    if ((header.flags & RTP_LARGE_FRAME) != 0 && header.offset_full >= header.data_length_full) {
         LOGGER_ERROR(m->log, "Invalid video packet: frame offset (%u) >= full frame length (%u)",
                      (unsigned)header.offset_full, (unsigned)header.data_length_full);
         return -1;
@@ -483,11 +482,11 @@ static int handle_rtp_packet(Messenger *m, uint32_t friendnumber, const uint8_t
         return -1;
     }
 
-    LOGGER_DEBUG(m->log, "header.pt %d, video %d", (uint8_t)header.pt, (RTP_TYPE_VIDEO % 128));
+    LOGGER_DEBUG(m->log, "header.pt %d, video %d", (uint8_t)header.pt, RTP_TYPE_VIDEO % 128);
 
     // The sender uses the new large-frame capable protocol and is sending a
     // video packet.
-    if ((header.flags & RTP_LARGE_FRAME) && header.pt == (RTP_TYPE_VIDEO % 128)) {
+    if ((header.flags & RTP_LARGE_FRAME) != 0 && header.pt == (RTP_TYPE_VIDEO % 128)) {
         return handle_video_packet(session, &header, data + RTP_HEADER_SIZE, length - RTP_HEADER_SIZE, m->log);
     }
 
@@ -502,7 +501,7 @@ static int handle_rtp_packet(Messenger *m, uint32_t friendnumber, const uint8_t
         bwc_add_recv(session->bwc, length);
 
         /* Invoke processing of active multiparted message */
-        if (session->mp) {
+        if (session->mp != nullptr) {
             session->mcb(session->m->mono_time, session->cs, session->mp);
             session->mp = nullptr;
         }
@@ -516,7 +515,7 @@ static int handle_rtp_packet(Messenger *m, uint32_t friendnumber, const uint8_t
 
     /* The message is sent in multiple parts */
 
-    if (session->mp) {
+    if (session->mp != nullptr) {
         /* There are 2 possible situations in this case:
          *      1) being that we got the part of already processing message.
          *      2) being that we got the part of a new/old message.
@@ -658,7 +657,7 @@ RTPSession *rtp_new(int payload_type, Messenger *m, Tox *tox, uint32_t friendnum
 
     RTPSession *session = (RTPSession *)calloc(1, sizeof(RTPSession));
 
-    if (!session) {
+    if (session == nullptr) {
         LOGGER_WARNING(m->log, "Alloc failed! Program might misbehave!");
         return nullptr;
     }
@@ -674,7 +673,7 @@ RTPSession *rtp_new(int payload_type, Messenger *m, Tox *tox, uint32_t friendnum
     // First entry is free.
     session->work_buffer_list->next_free_entry = 0;
 
-    session->ssrc = payload_type == RTP_TYPE_VIDEO ? 0 : random_u32();
+    session->ssrc = payload_type == RTP_TYPE_VIDEO ? 0 : random_u32(m->rng);
     session->payload_type = payload_type;
     session->m = m;
     session->tox = tox;
@@ -701,7 +700,7 @@ RTPSession *rtp_new(int payload_type, Messenger *m, Tox *tox, uint32_t friendnum
 
 void rtp_kill(RTPSession *session)
 {
-    if (!session) {
+    if (session == nullptr) {
         return;
     }
 
@@ -711,6 +710,9 @@ void rtp_kill(RTPSession *session)
     LOGGER_DEBUG(session->m->log, "Terminated RTP session V3 work_buffer_list->next_free_entry: %d",
                  (int)session->work_buffer_list->next_free_entry);
 
+    for (int8_t i = 0; i < session->work_buffer_list->next_free_entry; ++i) {
+        free(session->work_buffer_list->work_buffer[i].buf);
+    }
     free(session->work_buffer_list);
     free(session);
 }
@@ -744,13 +746,18 @@ int rtp_stop_receiving(RTPSession *session)
 }
 
 /**
- * @param data is raw vpx data.
- * @param length is the length of the raw data.
+ * Send a frame of audio or video data, chunked in @ref RTPMessage instances.
+ *
+ * @param session The A/V session to send the data for.
+ * @param data A byte array of length @p length.
+ * @param length The number of bytes to send from @p data.
+ * @param is_keyframe Whether this video frame is a key frame. If it is an
+ *   audio frame, this parameter is ignored.
  */
 int rtp_send_data(RTPSession *session, const uint8_t *data, uint32_t length,
                   bool is_keyframe, const Logger *log)
 {
-    if (!session) {
+    if (session == nullptr) {
         LOGGER_ERROR(log, "No session!");
         return -1;
     }
@@ -804,7 +811,7 @@ int rtp_send_data(RTPSession *session, const uint8_t *data, uint32_t length,
     rdata[0] = session->payload_type;  // packet id == payload_type
 
     if (MAX_CRYPTO_DATA_SIZE > (length + RTP_HEADER_SIZE + 1)) {
-        /**
+        /*
          * The length is lesser than the maximum allowed length (including header)
          * Send the packet in single piece.
          */
@@ -812,13 +819,13 @@ int rtp_send_data(RTPSession *session, const uint8_t *data, uint32_t length,
         memcpy(rdata + 1 + RTP_HEADER_SIZE, data, length);
 
         if (-1 == rtp_send_custom_lossy_packet(session->tox, session->friend_number, rdata, SIZEOF_VLA(rdata))) {
-            const char *netstrerror = net_new_strerror(net_error());
-            LOGGER_WARNING(session->m->log, "RTP send failed (len: %u)! std error: %s, net error: %s",
-                           (unsigned)SIZEOF_VLA(rdata), strerror(errno), netstrerror);
+            char *netstrerror = net_new_strerror(net_error());
+            LOGGER_WARNING(session->m->log, "RTP send failed (len: %u)! net error: %s",
+                           (unsigned)SIZEOF_VLA(rdata), netstrerror);
             net_kill_strerror(netstrerror);
         }
     } else {
-        /**
+        /*
          * The length is greater than the maximum allowed length (including header)
          * Send the packet in multiple pieces.
          */
@@ -831,9 +838,9 @@ int rtp_send_data(RTPSession *session, const uint8_t *data, uint32_t length,
 
             if (-1 == rtp_send_custom_lossy_packet(session->tox, session->friend_number,
                                                    rdata, piece + RTP_HEADER_SIZE + 1)) {
-                const char *netstrerror = net_new_strerror(net_error());
-                LOGGER_WARNING(session->m->log, "RTP send failed (len: %d)! std error: %s, net error: %s",
-                               piece + RTP_HEADER_SIZE + 1, strerror(errno), netstrerror);
+                char *netstrerror = net_new_strerror(net_error());
+                LOGGER_WARNING(session->m->log, "RTP send failed (len: %d)! net error: %s",
+                               piece + RTP_HEADER_SIZE + 1, netstrerror);
                 net_kill_strerror(netstrerror);
             }
 
@@ -845,15 +852,15 @@ int rtp_send_data(RTPSession *session, const uint8_t *data, uint32_t length,
         /* Send remaining */
         piece = length - sent;
 
-        if (piece) {
+        if (piece != 0) {
             rtp_header_pack(rdata + 1, &header);
             memcpy(rdata + 1 + RTP_HEADER_SIZE, data + sent, piece);
 
             if (-1 == rtp_send_custom_lossy_packet(session->tox, session->friend_number, rdata,
                                                    piece + RTP_HEADER_SIZE + 1)) {
-                const char *netstrerror = net_new_strerror(net_error());
-                LOGGER_WARNING(session->m->log, "RTP send failed (len: %d)! std error: %s, net error: %s",
-                               piece + RTP_HEADER_SIZE + 1, strerror(errno), netstrerror);
+                char *netstrerror = net_new_strerror(net_error());
+                LOGGER_WARNING(session->m->log, "RTP send failed (len: %d)! net error: %s",
+                               piece + RTP_HEADER_SIZE + 1, netstrerror);
                 net_kill_strerror(netstrerror);
             }
         }
