@@ -2,26 +2,24 @@
  * Copyright © 2016-2018 The TokTok team.
  * Copyright © 2013-2015 Tox project.
  */
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif /* HAVE_CONFIG_H */
-
 #include "toxav.h"
 
-#include "msi.h"
-#include "rtp.h"
-
-#include "../toxcore/Messenger.h"
-#include "../toxcore/logger.h"
-#include "../toxcore/mono_time.h"
-#include "../toxcore/util.h"
-
 #include <assert.h>
 #include <errno.h>
 #include <limits.h>
 #include <stdlib.h>
 #include <string.h>
 
+#include "msi.h"
+#include "rtp.h"
+
+#include "../toxcore/Messenger.h"
+#include "../toxcore/ccompat.h"
+#include "../toxcore/logger.h"
+#include "../toxcore/mono_time.h"
+#include "../toxcore/tox_struct.h"
+#include "../toxcore/util.h"
+
 // TODO(zoff99): don't hardcode this, let the application choose it
 // VPX Info: Time to spend encoding, in microseconds (it's a *soft* deadline)
 #define WANTED_MAX_ENCODER_FPS 40
@@ -35,7 +33,10 @@
  * VPX_DL_BEST_QUALITY   (0)       deadline parameter analogous to VPx BEST QUALITY mode.
  */
 
-typedef struct ToxAVCall_s {
+// iteration interval that is used when no call is active
+#define IDLE_ITERATION_INTERVAL_MS 200
+
+typedef struct ToxAVCall {
     ToxAV *av;
 
     pthread_mutex_t mutex_audio[1];
@@ -60,10 +61,24 @@ typedef struct ToxAVCall_s {
 
     pthread_mutex_t toxav_call_mutex[1];
 
-    struct ToxAVCall_s *prev;
-    struct ToxAVCall_s *next;
+    struct ToxAVCall *prev;
+    struct ToxAVCall *next;
 } ToxAVCall;
 
+
+/** Decode time statistics */
+typedef struct DecodeTimeStats {
+    /** Measure count */
+    int32_t count;
+    /** Last cycle total */
+    int32_t total;
+    /** Average decoding time in ms */
+    int32_t average;
+
+    /** Calculated iteration interval */
+    uint32_t interval;
+} DecodeTimeStats;
+
 struct ToxAV {
     Tox *tox;
     Messenger *m;
@@ -94,13 +109,11 @@ struct ToxAV {
     toxav_video_bit_rate_cb *vbcb;
     void *vbcb_user_data;
 
-    /** Decode time measures */
-    int32_t dmssc; /** Measure count */
-    int32_t dmsst; /** Last cycle total */
-    int32_t dmssa; /** Average decoding time in ms */
-
-    uint32_t interval; /** Calculated interval */
-    Mono_Time *toxav_mono_time; /** ToxAV's own mono_time instance */
+    /* keep track of decode times for audio and video */
+    DecodeTimeStats audio_stats;
+    DecodeTimeStats video_stats;
+    /** ToxAV's own mono_time instance */
+    Mono_Time *toxav_mono_time;
 };
 
 static void callback_bwc(BWController *bwc, uint32_t friend_number, float loss, void *user_data);
@@ -120,6 +133,19 @@ static ToxAVCall *call_remove(ToxAVCall *call);
 static bool call_prepare_transmission(ToxAVCall *call);
 static void call_kill_transmission(ToxAVCall *call);
 
+/**
+ * @brief initialize d with default values
+ * @param d struct to be initialized, must not be nullptr
+ */
+static void init_decode_time_stats(DecodeTimeStats *d)
+{
+    assert(d != nullptr);
+    d->count = 0;
+    d->total = 0;
+    d->average = 0;
+    d->interval = IDLE_ITERATION_INTERVAL_MS;
+}
+
 ToxAV *toxav_new(Tox *tox, Toxav_Err_New *error)
 {
     Toxav_Err_New rc = TOXAV_ERR_NEW_OK;
@@ -132,16 +158,14 @@ ToxAV *toxav_new(Tox *tox, Toxav_Err_New *error)
 
     // TODO(iphydf): Don't rely on toxcore internals.
     Messenger *m;
-    //!TOKSTYLE-
-    m = *(Messenger **)tox;
-    //!TOKSTYLE+
+    m = tox->m;
 
-    if (m->msi_packet) {
+    if (m->msi_packet != nullptr) {
         rc = TOXAV_ERR_NEW_MULTIPLE;
         goto RETURN;
     }
 
-    av = (ToxAV *)calloc(sizeof(ToxAV), 1);
+    av = (ToxAV *)calloc(1, sizeof(ToxAV));
 
     if (av == nullptr) {
         LOGGER_WARNING(m->log, "Allocation failed!");
@@ -157,7 +181,7 @@ ToxAV *toxav_new(Tox *tox, Toxav_Err_New *error)
 
     av->tox = tox;
     av->m = m;
-    av->toxav_mono_time = mono_time_new();
+    av->toxav_mono_time = mono_time_new(nullptr, nullptr);
     av->msi = msi_new(av->m);
 
     if (av->msi == nullptr) {
@@ -166,19 +190,20 @@ ToxAV *toxav_new(Tox *tox, Toxav_Err_New *error)
         goto RETURN;
     }
 
-    av->interval = 200;
+    init_decode_time_stats(&av->audio_stats);
+    init_decode_time_stats(&av->video_stats);
     av->msi->av = av;
 
-    msi_register_callback(av->msi, callback_invite, MSI_ON_INVITE);
-    msi_register_callback(av->msi, callback_start, MSI_ON_START);
-    msi_register_callback(av->msi, callback_end, MSI_ON_END);
-    msi_register_callback(av->msi, callback_error, MSI_ON_ERROR);
-    msi_register_callback(av->msi, callback_error, MSI_ON_PEERTIMEOUT);
-    msi_register_callback(av->msi, callback_capabilites, MSI_ON_CAPABILITIES);
+    msi_callback_invite(av->msi, callback_invite);
+    msi_callback_start(av->msi, callback_start);
+    msi_callback_end(av->msi, callback_end);
+    msi_callback_error(av->msi, callback_error);
+    msi_callback_peertimeout(av->msi, callback_error);
+    msi_callback_capabilities(av->msi, callback_capabilites);
 
 RETURN:
 
-    if (error) {
+    if (error != nullptr) {
         *error = rc;
     }
 
@@ -198,16 +223,16 @@ void toxav_kill(ToxAV *av)
     pthread_mutex_lock(av->mutex);
 
     /* To avoid possible deadlocks */
-    while (av->msi && msi_kill(av->msi, av->m->log) != 0) {
+    while (av->msi != nullptr && msi_kill(av->msi, av->m->log) != 0) {
         pthread_mutex_unlock(av->mutex);
         pthread_mutex_lock(av->mutex);
     }
 
     /* Msi kill will hang up all calls so just clean these calls */
-    if (av->calls) {
+    if (av->calls != nullptr) {
         ToxAVCall *it = call_get(av, av->calls_head);
 
-        while (it) {
+        while (it != nullptr) {
             call_kill_transmission(it);
             it->msi_call = nullptr; /* msi_kill() frees the call's msi_call handle; which causes #278 */
             it = call_remove(it); /* This will eventually free av->calls */
@@ -225,12 +250,48 @@ Tox *toxav_get_tox(const ToxAV *av)
 {
     return av->tox;
 }
+
+uint32_t toxav_audio_iteration_interval(const ToxAV *av)
+{
+    return av->calls != nullptr ? av->audio_stats.interval : IDLE_ITERATION_INTERVAL_MS;
+}
+
+uint32_t toxav_video_iteration_interval(const ToxAV *av)
+{
+    return av->calls != nullptr ? av->video_stats.interval : IDLE_ITERATION_INTERVAL_MS;
+}
+
 uint32_t toxav_iteration_interval(const ToxAV *av)
 {
-    /* If no call is active interval is 200 */
-    return av->calls ? av->interval : 200;
+    return min_u32(toxav_audio_iteration_interval(av),
+                   toxav_video_iteration_interval(av));
 }
-void toxav_iterate(ToxAV *av)
+
+/**
+ * @brief calc_interval Calculates the needed iteration interval based on previous decode times
+ * @param av ToxAV struct to work on
+ * @param stats Statistics to update
+ * @param frame_time the duration of the current frame in ms
+ * @param start_time the timestamp when decoding of this frame started
+ */
+static void calc_interval(ToxAV *av, DecodeTimeStats *stats, int32_t frame_time, uint64_t start_time)
+{
+    stats->interval = frame_time < stats->average ? 0 : (frame_time - stats->average);
+    stats->total += current_time_monotonic(av->m->mono_time) - start_time;
+
+    if (++stats->count == 3) {
+        stats->average = stats->total / 3 + 5; /* NOTE: Magic Offset for precision */
+        stats->count = 0;
+        stats->total = 0;
+    }
+}
+
+/**
+ * @brief common iterator function for audio and video calls
+ * @param av pointer to ToxAV structure of current instance
+ * @param audio if true, iterate audio, video else
+ */
+static void iterate_common(ToxAV *av, bool audio)
 {
     pthread_mutex_lock(av->mutex);
 
@@ -239,54 +300,67 @@ void toxav_iterate(ToxAV *av)
         return;
     }
 
-    uint64_t start = current_time_monotonic(av->toxav_mono_time);
-    int32_t rc = 500;
+    const uint64_t start = current_time_monotonic(av->toxav_mono_time);
+    // time until the first audio or video frame is over
+    int32_t frame_time = IDLE_ITERATION_INTERVAL_MS;
 
-    ToxAVCall *i = av->calls[av->calls_head];
+    for (ToxAVCall *i = av->calls[av->calls_head]; i != nullptr; i = i->next) {
+        if (!i->active) {
+            continue;
+        }
 
-    for (; i; i = i->next) {
-        if (i->active) {
-            pthread_mutex_lock(i->toxav_call_mutex);
-            pthread_mutex_unlock(av->mutex);
+        pthread_mutex_lock(i->toxav_call_mutex);
+        pthread_mutex_unlock(av->mutex);
 
+        if (audio) {
             ac_iterate(i->audio);
+
+            if ((i->msi_call->self_capabilities & MSI_CAP_R_AUDIO) != 0 &&
+                    (i->msi_call->peer_capabilities & MSI_CAP_S_AUDIO) != 0) {
+                frame_time = min_s32(i->audio->lp_frame_duration, frame_time);
+            }
+        } else {
             vc_iterate(i->video);
 
-            if (i->msi_call->self_capabilities & MSI_CAP_R_AUDIO &&
-                    i->msi_call->peer_capabilities & MSI_CAP_S_AUDIO) {
-                rc = min_s32(i->audio->lp_frame_duration, rc);
-            }
-
-            if (i->msi_call->self_capabilities & MSI_CAP_R_VIDEO &&
-                    i->msi_call->peer_capabilities & MSI_CAP_S_VIDEO) {
+            if ((i->msi_call->self_capabilities & MSI_CAP_R_VIDEO) != 0 &&
+                    (i->msi_call->peer_capabilities & MSI_CAP_S_VIDEO) != 0) {
                 pthread_mutex_lock(i->video->queue_mutex);
-                rc = min_u32(i->video->lcfd, rc);
+                frame_time = min_s32(i->video->lcfd, frame_time);
                 pthread_mutex_unlock(i->video->queue_mutex);
             }
+        }
 
-            uint32_t fid = i->friend_number;
+        const uint32_t fid = i->friend_number;
 
-            pthread_mutex_unlock(i->toxav_call_mutex);
-            pthread_mutex_lock(av->mutex);
+        pthread_mutex_unlock(i->toxav_call_mutex);
+        pthread_mutex_lock(av->mutex);
 
-            /* In case this call is popped from container stop iteration */
-            if (call_get(av, fid) != i) {
-                break;
-            }
+        /* In case this call is popped from container stop iteration */
+        if (call_get(av, fid) != i) {
+            break;
         }
     }
 
-    av->interval = rc < av->dmssa ? 0 : (rc - av->dmssa);
-    av->dmsst += current_time_monotonic(av->toxav_mono_time) - start;
-
-    if (++av->dmssc == 3) {
-        av->dmssa = av->dmsst / 3 + 5; /* NOTE Magic Offset 5 for precision */
-        av->dmssc = 0;
-        av->dmsst = 0;
-    }
-
+    DecodeTimeStats *stats = audio ? &av->audio_stats : &av->video_stats;
+    calc_interval(av, stats, frame_time, start);
     pthread_mutex_unlock(av->mutex);
 }
+void toxav_audio_iterate(ToxAV *av)
+{
+    iterate_common(av, true);
+}
+
+void toxav_video_iterate(ToxAV *av)
+{
+    iterate_common(av, false);
+}
+
+void toxav_iterate(ToxAV *av)
+{
+    toxav_audio_iterate(av);
+    toxav_video_iterate(av);
+}
+
 bool toxav_call(ToxAV *av, uint32_t friend_number, uint32_t audio_bit_rate, uint32_t video_bit_rate,
                 Toxav_Err_Call *error)
 {
@@ -295,8 +369,8 @@ bool toxav_call(ToxAV *av, uint32_t friend_number, uint32_t audio_bit_rate, uint
 
     pthread_mutex_lock(av->mutex);
 
-    if ((audio_bit_rate && audio_bit_rate_invalid(audio_bit_rate))
-            || (video_bit_rate && video_bit_rate_invalid(video_bit_rate))) {
+    if ((audio_bit_rate != 0 && audio_bit_rate_invalid(audio_bit_rate))
+            || (video_bit_rate != 0 && video_bit_rate_invalid(video_bit_rate))) {
         rc = TOXAV_ERR_CALL_INVALID_BIT_RATE;
         goto RETURN;
     }
@@ -326,7 +400,7 @@ bool toxav_call(ToxAV *av, uint32_t friend_number, uint32_t audio_bit_rate, uint
 RETURN:
     pthread_mutex_unlock(av->mutex);
 
-    if (error) {
+    if (error != nullptr) {
         *error = rc;
     }
 
@@ -347,13 +421,13 @@ bool toxav_answer(ToxAV *av, uint32_t friend_number, uint32_t audio_bit_rate, ui
     Toxav_Err_Answer rc = TOXAV_ERR_ANSWER_OK;
     ToxAVCall *call;
 
-    if (m_friend_exists(av->m, friend_number) == 0) {
+    if (!m_friend_exists(av->m, friend_number)) {
         rc = TOXAV_ERR_ANSWER_FRIEND_NOT_FOUND;
         goto RETURN;
     }
 
-    if ((audio_bit_rate && audio_bit_rate_invalid(audio_bit_rate))
-            || (video_bit_rate && video_bit_rate_invalid(video_bit_rate))
+    if ((audio_bit_rate != 0 && audio_bit_rate_invalid(audio_bit_rate))
+            || (video_bit_rate != 0 && video_bit_rate_invalid(video_bit_rate))
        ) {
         rc = TOXAV_ERR_ANSWER_INVALID_BIT_RATE;
         goto RETURN;
@@ -386,7 +460,7 @@ bool toxav_answer(ToxAV *av, uint32_t friend_number, uint32_t audio_bit_rate, ui
 RETURN:
     pthread_mutex_unlock(av->mutex);
 
-    if (error) {
+    if (error != nullptr) {
         *error = rc;
     }
 
@@ -399,169 +473,183 @@ void toxav_callback_call_state(ToxAV *av, toxav_call_state_cb *callback, void *u
     av->scb_user_data = user_data;
     pthread_mutex_unlock(av->mutex);
 }
+static Toxav_Err_Call_Control call_control_handle_resume(const ToxAVCall *call)
+{
+    /* Only act if paused and had media transfer active before */
+    if (call->msi_call->self_capabilities != 0 || call->previous_self_capabilities == 0) {
+        return TOXAV_ERR_CALL_CONTROL_INVALID_TRANSITION;
+    }
+
+    if (msi_change_capabilities(call->msi_call, call->previous_self_capabilities) == -1) {
+        return TOXAV_ERR_CALL_CONTROL_SYNC;
+    }
+
+    rtp_allow_receiving(call->audio_rtp);
+    rtp_allow_receiving(call->video_rtp);
+
+    return TOXAV_ERR_CALL_CONTROL_OK;
+}
+static Toxav_Err_Call_Control call_control_handle_pause(ToxAVCall *call)
+{
+    /* Only act if not already paused */
+    if (call->msi_call->self_capabilities == 0) {
+        return TOXAV_ERR_CALL_CONTROL_INVALID_TRANSITION;
+    }
+
+    call->previous_self_capabilities = call->msi_call->self_capabilities;
+
+    if (msi_change_capabilities(call->msi_call, 0) == -1) {
+        return TOXAV_ERR_CALL_CONTROL_SYNC;
+    }
+
+    rtp_stop_receiving(call->audio_rtp);
+    rtp_stop_receiving(call->video_rtp);
+
+    return TOXAV_ERR_CALL_CONTROL_OK;
+}
+static Toxav_Err_Call_Control call_control_handle_cancel(ToxAVCall *call)
+{
+    /* Hang up */
+    pthread_mutex_lock(call->toxav_call_mutex);
+
+    if (msi_hangup(call->msi_call) != 0) {
+        pthread_mutex_unlock(call->toxav_call_mutex);
+        return TOXAV_ERR_CALL_CONTROL_SYNC;
+    }
+
+    call->msi_call = nullptr;
+    pthread_mutex_unlock(call->toxav_call_mutex);
+
+    /* No matter the case, terminate the call */
+    call_kill_transmission(call);
+    call_remove(call);
+
+    return TOXAV_ERR_CALL_CONTROL_OK;
+}
+static Toxav_Err_Call_Control call_control_handle_mute_audio(const ToxAVCall *call)
+{
+    if ((call->msi_call->self_capabilities & MSI_CAP_R_AUDIO) == 0) {
+        return TOXAV_ERR_CALL_CONTROL_INVALID_TRANSITION;
+    }
+
+    if (msi_change_capabilities(call->msi_call, call->
+                                msi_call->self_capabilities ^ MSI_CAP_R_AUDIO) == -1) {
+        return TOXAV_ERR_CALL_CONTROL_SYNC;
+
+    }
+
+    rtp_stop_receiving(call->audio_rtp);
+    return TOXAV_ERR_CALL_CONTROL_OK;
+}
+static Toxav_Err_Call_Control call_control_handle_unmute_audio(const ToxAVCall *call)
+{
+    if ((call->msi_call->self_capabilities ^ MSI_CAP_R_AUDIO) == 0) {
+        return TOXAV_ERR_CALL_CONTROL_INVALID_TRANSITION;
+    }
+
+    if (msi_change_capabilities(call->msi_call, call->
+                                msi_call->self_capabilities | MSI_CAP_R_AUDIO) == -1) {
+        return TOXAV_ERR_CALL_CONTROL_SYNC;
+    }
+
+    rtp_allow_receiving(call->audio_rtp);
+    return TOXAV_ERR_CALL_CONTROL_OK;
+}
+static Toxav_Err_Call_Control call_control_handle_hide_video(const ToxAVCall *call)
+{
+    if ((call->msi_call->self_capabilities & MSI_CAP_R_VIDEO) == 0) {
+        return TOXAV_ERR_CALL_CONTROL_INVALID_TRANSITION;
+    }
+
+    if (msi_change_capabilities(call->msi_call, call->
+                                msi_call->self_capabilities ^ MSI_CAP_R_VIDEO) == -1) {
+        return TOXAV_ERR_CALL_CONTROL_SYNC;
+    }
+
+    rtp_stop_receiving(call->video_rtp);
+    return TOXAV_ERR_CALL_CONTROL_OK;
+}
+static Toxav_Err_Call_Control call_control_handle_show_video(const ToxAVCall *call)
+{
+    if ((call->msi_call->self_capabilities ^ MSI_CAP_R_VIDEO) == 0) {
+        return TOXAV_ERR_CALL_CONTROL_INVALID_TRANSITION;
+    }
+
+    if (msi_change_capabilities(call->msi_call, call->
+                                msi_call->self_capabilities | MSI_CAP_R_VIDEO) == -1) {
+        return TOXAV_ERR_CALL_CONTROL_SYNC;
+    }
+
+    rtp_allow_receiving(call->video_rtp);
+    return TOXAV_ERR_CALL_CONTROL_OK;
+}
+static Toxav_Err_Call_Control call_control_handle(ToxAVCall *call, Toxav_Call_Control control)
+{
+    switch (control) {
+        case TOXAV_CALL_CONTROL_RESUME:
+            return call_control_handle_resume(call);
+
+        case TOXAV_CALL_CONTROL_PAUSE:
+            return call_control_handle_pause(call);
+
+        case TOXAV_CALL_CONTROL_CANCEL:
+            return call_control_handle_cancel(call);
+
+        case TOXAV_CALL_CONTROL_MUTE_AUDIO:
+            return call_control_handle_mute_audio(call);
+
+        case TOXAV_CALL_CONTROL_UNMUTE_AUDIO:
+            return call_control_handle_unmute_audio(call);
+
+        case TOXAV_CALL_CONTROL_HIDE_VIDEO:
+            return call_control_handle_hide_video(call);
+
+        case TOXAV_CALL_CONTROL_SHOW_VIDEO:
+            return call_control_handle_show_video(call);
+    }
+
+    return TOXAV_ERR_CALL_CONTROL_INVALID_TRANSITION;
+}
+static Toxav_Err_Call_Control call_control(ToxAV *av, uint32_t friend_number, Toxav_Call_Control control)
+{
+    if (!m_friend_exists(av->m, friend_number)) {
+        return TOXAV_ERR_CALL_CONTROL_FRIEND_NOT_FOUND;
+    }
+
+    ToxAVCall *call = call_get(av, friend_number);
+
+    if (call == nullptr || (!call->active && control != TOXAV_CALL_CONTROL_CANCEL)) {
+        return TOXAV_ERR_CALL_CONTROL_FRIEND_NOT_IN_CALL;
+    }
+
+    return call_control_handle(call, control);
+}
 bool toxav_call_control(ToxAV *av, uint32_t friend_number, Toxav_Call_Control control, Toxav_Err_Call_Control *error)
 {
     pthread_mutex_lock(av->mutex);
-    Toxav_Err_Call_Control rc = TOXAV_ERR_CALL_CONTROL_OK;
-    ToxAVCall *call;
 
-    if (m_friend_exists(av->m, friend_number) == 0) {
-        rc = TOXAV_ERR_CALL_CONTROL_FRIEND_NOT_FOUND;
-        goto RETURN;
-    }
+    const Toxav_Err_Call_Control rc = call_control(av, friend_number, control);
 
-    call = call_get(av, friend_number);
-
-    if (call == nullptr || (!call->active && control != TOXAV_CALL_CONTROL_CANCEL)) {
-        rc = TOXAV_ERR_CALL_CONTROL_FRIEND_NOT_IN_CALL;
-        goto RETURN;
-    }
-
-    switch (control) {
-        case TOXAV_CALL_CONTROL_RESUME: {
-            /* Only act if paused and had media transfer active before */
-            if (call->msi_call->self_capabilities == 0 &&
-                    call->previous_self_capabilities) {
-
-                if (msi_change_capabilities(call->msi_call,
-                                            call->previous_self_capabilities) == -1) {
-                    rc = TOXAV_ERR_CALL_CONTROL_SYNC;
-                    goto RETURN;
-                }
-
-                rtp_allow_receiving(call->audio_rtp);
-                rtp_allow_receiving(call->video_rtp);
-            } else {
-                rc = TOXAV_ERR_CALL_CONTROL_INVALID_TRANSITION;
-                goto RETURN;
-            }
-        }
-        break;
-
-        case TOXAV_CALL_CONTROL_PAUSE: {
-            /* Only act if not already paused */
-            if (call->msi_call->self_capabilities) {
-                call->previous_self_capabilities = call->msi_call->self_capabilities;
-
-                if (msi_change_capabilities(call->msi_call, 0) == -1) {
-                    rc = TOXAV_ERR_CALL_CONTROL_SYNC;
-                    goto RETURN;
-                }
-
-                rtp_stop_receiving(call->audio_rtp);
-                rtp_stop_receiving(call->video_rtp);
-            } else {
-                rc = TOXAV_ERR_CALL_CONTROL_INVALID_TRANSITION;
-                goto RETURN;
-            }
-        }
-        break;
-
-        case TOXAV_CALL_CONTROL_CANCEL: {
-            /* Hang up */
-            pthread_mutex_lock(call->toxav_call_mutex);
-
-            if (msi_hangup(call->msi_call) != 0) {
-                rc = TOXAV_ERR_CALL_CONTROL_SYNC;
-                pthread_mutex_unlock(call->toxav_call_mutex);
-                goto RETURN;
-            }
-
-            call->msi_call = nullptr;
-            pthread_mutex_unlock(call->toxav_call_mutex);
-
-            /* No mather the case, terminate the call */
-            call_kill_transmission(call);
-            call_remove(call);
-        }
-        break;
-
-        case TOXAV_CALL_CONTROL_MUTE_AUDIO: {
-            if (call->msi_call->self_capabilities & MSI_CAP_R_AUDIO) {
-                if (msi_change_capabilities(call->msi_call, call->
-                                            msi_call->self_capabilities ^ MSI_CAP_R_AUDIO) == -1) {
-                    rc = TOXAV_ERR_CALL_CONTROL_SYNC;
-                    goto RETURN;
-                }
-
-                rtp_stop_receiving(call->audio_rtp);
-            } else {
-                rc = TOXAV_ERR_CALL_CONTROL_INVALID_TRANSITION;
-                goto RETURN;
-            }
-        }
-        break;
-
-        case TOXAV_CALL_CONTROL_UNMUTE_AUDIO: {
-            if (call->msi_call->self_capabilities ^ MSI_CAP_R_AUDIO) {
-                if (msi_change_capabilities(call->msi_call, call->
-                                            msi_call->self_capabilities | MSI_CAP_R_AUDIO) == -1) {
-                    rc = TOXAV_ERR_CALL_CONTROL_SYNC;
-                    goto RETURN;
-                }
-
-                rtp_allow_receiving(call->audio_rtp);
-            } else {
-                rc = TOXAV_ERR_CALL_CONTROL_INVALID_TRANSITION;
-                goto RETURN;
-            }
-        }
-        break;
-
-        case TOXAV_CALL_CONTROL_HIDE_VIDEO: {
-            if (call->msi_call->self_capabilities & MSI_CAP_R_VIDEO) {
-                if (msi_change_capabilities(call->msi_call, call->
-                                            msi_call->self_capabilities ^ MSI_CAP_R_VIDEO) == -1) {
-                    rc = TOXAV_ERR_CALL_CONTROL_SYNC;
-                    goto RETURN;
-                }
-
-                rtp_stop_receiving(call->video_rtp);
-            } else {
-                rc = TOXAV_ERR_CALL_CONTROL_INVALID_TRANSITION;
-                goto RETURN;
-            }
-        }
-        break;
-
-        case TOXAV_CALL_CONTROL_SHOW_VIDEO: {
-            if (call->msi_call->self_capabilities ^ MSI_CAP_R_VIDEO) {
-                if (msi_change_capabilities(call->msi_call, call->
-                                            msi_call->self_capabilities | MSI_CAP_R_VIDEO) == -1) {
-                    rc = TOXAV_ERR_CALL_CONTROL_SYNC;
-                    goto RETURN;
-                }
-
-                rtp_allow_receiving(call->video_rtp);
-            } else {
-                rc = TOXAV_ERR_CALL_CONTROL_INVALID_TRANSITION;
-                goto RETURN;
-            }
-        }
-        break;
-    }
-
-RETURN:
     pthread_mutex_unlock(av->mutex);
 
-    if (error) {
+    if (error != nullptr) {
         *error = rc;
     }
 
     return rc == TOXAV_ERR_CALL_CONTROL_OK;
 }
-bool toxav_audio_set_bit_rate(ToxAV *av, uint32_t friend_number, uint32_t audio_bit_rate,
+bool toxav_audio_set_bit_rate(ToxAV *av, uint32_t friend_number, uint32_t bit_rate,
                               Toxav_Err_Bit_Rate_Set *error)
 {
     Toxav_Err_Bit_Rate_Set rc = TOXAV_ERR_BIT_RATE_SET_OK;
     ToxAVCall *call;
 
-    if (m_friend_exists(av->m, friend_number) == 0) {
+    if (!m_friend_exists(av->m, friend_number)) {
         rc = TOXAV_ERR_BIT_RATE_SET_FRIEND_NOT_FOUND;
         goto RETURN;
     }
 
-    if (audio_bit_rate > 0 && audio_bit_rate_invalid(audio_bit_rate)) {
+    if (bit_rate > 0 && audio_bit_rate_invalid(bit_rate)) {
         rc = TOXAV_ERR_BIT_RATE_SET_INVALID_BIT_RATE;
         goto RETURN;
     }
@@ -575,11 +663,11 @@ bool toxav_audio_set_bit_rate(ToxAV *av, uint32_t friend_number, uint32_t audio_
         goto RETURN;
     }
 
-    LOGGER_DEBUG(av->m->log, "Setting new audio bitrate to: %d", audio_bit_rate);
+    LOGGER_DEBUG(av->m->log, "Setting new audio bitrate to: %d", bit_rate);
 
-    if (call->audio_bit_rate == audio_bit_rate) {
-        LOGGER_DEBUG(av->m->log, "Audio bitrate already set to: %d", audio_bit_rate);
-    } else if (audio_bit_rate == 0) {
+    if (call->audio_bit_rate == bit_rate) {
+        LOGGER_DEBUG(av->m->log, "Audio bitrate already set to: %d", bit_rate);
+    } else if (bit_rate == 0) {
         LOGGER_DEBUG(av->m->log, "Turned off audio sending");
 
         if (msi_change_capabilities(call->msi_call, call->msi_call->
@@ -606,34 +694,34 @@ bool toxav_audio_set_bit_rate(ToxAV *av, uint32_t friend_number, uint32_t audio_
                 goto RETURN;
             }
         } else {
-            LOGGER_DEBUG(av->m->log, "Set new audio bit rate %d", audio_bit_rate);
+            LOGGER_DEBUG(av->m->log, "Set new audio bit rate %d", bit_rate);
         }
 
-        call->audio_bit_rate = audio_bit_rate;
+        call->audio_bit_rate = bit_rate;
         pthread_mutex_unlock(call->toxav_call_mutex);
     }
 
     pthread_mutex_unlock(av->mutex);
 RETURN:
 
-    if (error) {
+    if (error != nullptr) {
         *error = rc;
     }
 
     return rc == TOXAV_ERR_BIT_RATE_SET_OK;
 }
-bool toxav_video_set_bit_rate(ToxAV *av, uint32_t friend_number, uint32_t video_bit_rate,
+bool toxav_video_set_bit_rate(ToxAV *av, uint32_t friend_number, uint32_t bit_rate,
                               Toxav_Err_Bit_Rate_Set *error)
 {
     Toxav_Err_Bit_Rate_Set rc = TOXAV_ERR_BIT_RATE_SET_OK;
     ToxAVCall *call;
 
-    if (m_friend_exists(av->m, friend_number) == 0) {
+    if (!m_friend_exists(av->m, friend_number)) {
         rc = TOXAV_ERR_BIT_RATE_SET_FRIEND_NOT_FOUND;
         goto RETURN;
     }
 
-    if (video_bit_rate > 0 && video_bit_rate_invalid(video_bit_rate)) {
+    if (bit_rate > 0 && video_bit_rate_invalid(bit_rate)) {
         rc = TOXAV_ERR_BIT_RATE_SET_INVALID_BIT_RATE;
         goto RETURN;
     }
@@ -647,11 +735,11 @@ bool toxav_video_set_bit_rate(ToxAV *av, uint32_t friend_number, uint32_t video_
         goto RETURN;
     }
 
-    LOGGER_DEBUG(av->m->log, "Setting new video bitrate to: %d", video_bit_rate);
+    LOGGER_DEBUG(av->m->log, "Setting new video bitrate to: %d", bit_rate);
 
-    if (call->video_bit_rate == video_bit_rate) {
-        LOGGER_DEBUG(av->m->log, "Video bitrate already set to: %d", video_bit_rate);
-    } else if (video_bit_rate == 0) {
+    if (call->video_bit_rate == bit_rate) {
+        LOGGER_DEBUG(av->m->log, "Video bitrate already set to: %d", bit_rate);
+    } else if (bit_rate == 0) {
         LOGGER_DEBUG(av->m->log, "Turned off video sending");
 
         /* Video sending is turned off; notify peer */
@@ -678,17 +766,17 @@ bool toxav_video_set_bit_rate(ToxAV *av, uint32_t friend_number, uint32_t video_
                 goto RETURN;
             }
         } else {
-            LOGGER_DEBUG(av->m->log, "Set new video bit rate %d", video_bit_rate);
+            LOGGER_DEBUG(av->m->log, "Set new video bit rate %d", bit_rate);
         }
 
-        call->video_bit_rate = video_bit_rate;
+        call->video_bit_rate = bit_rate;
         pthread_mutex_unlock(call->toxav_call_mutex);
     }
 
     pthread_mutex_unlock(av->mutex);
 RETURN:
 
-    if (error) {
+    if (error != nullptr) {
         *error = rc;
     }
 
@@ -714,7 +802,7 @@ bool toxav_audio_send_frame(ToxAV *av, uint32_t friend_number, const int16_t *pc
     Toxav_Err_Send_Frame rc = TOXAV_ERR_SEND_FRAME_OK;
     ToxAVCall *call;
 
-    if (m_friend_exists(av->m, friend_number) == 0) {
+    if (!m_friend_exists(av->m, friend_number)) {
         rc = TOXAV_ERR_SEND_FRAME_FRIEND_NOT_FOUND;
         goto RETURN;
     }
@@ -733,8 +821,8 @@ bool toxav_audio_send_frame(ToxAV *av, uint32_t friend_number, const int16_t *pc
     }
 
     if (call->audio_bit_rate == 0 ||
-            !(call->msi_call->self_capabilities & MSI_CAP_S_AUDIO) ||
-            !(call->msi_call->peer_capabilities & MSI_CAP_R_AUDIO)) {
+            (call->msi_call->self_capabilities & MSI_CAP_S_AUDIO) == 0 ||
+            (call->msi_call->peer_capabilities & MSI_CAP_R_AUDIO) == 0) {
         pthread_mutex_unlock(av->mutex);
         rc = TOXAV_ERR_SEND_FRAME_PAYLOAD_TYPE_DISABLED;
         goto RETURN;
@@ -766,7 +854,7 @@ bool toxav_audio_send_frame(ToxAV *av, uint32_t friend_number, const int16_t *pc
 
         sampling_rate = net_htonl(sampling_rate);
         memcpy(dest, &sampling_rate, sizeof(sampling_rate));
-        int vrc = opus_encode(call->audio->encoder, pcm, sample_count,
+        const int vrc = opus_encode(call->audio->encoder, pcm, sample_count,
                               dest + sizeof(sampling_rate), SIZEOF_VLA(dest) - sizeof(sampling_rate));
 
         if (vrc < 0) {
@@ -786,7 +874,7 @@ bool toxav_audio_send_frame(ToxAV *av, uint32_t friend_number, const int16_t *pc
 
 RETURN:
 
-    if (error) {
+    if (error != nullptr) {
         *error = rc;
     }
 
@@ -823,7 +911,9 @@ static Toxav_Err_Send_Frame send_frames(const Logger *log, ToxAVCall *call)
         LOGGER_DEBUG(log, "+ _sending_FRAME_ b0=%d b1=%d", buf[0], buf[1]);
 
         if (res < 0) {
-            LOGGER_WARNING(log, "Could not send video frame: %s", strerror(errno));
+            char *netstrerror = net_new_strerror(net_error());
+            LOGGER_WARNING(log, "Could not send video frame: %s", netstrerror);
+            net_kill_strerror(netstrerror);
             return TOXAV_ERR_SEND_FRAME_RTP_FAILED;
         }
     }
@@ -839,7 +929,7 @@ bool toxav_video_send_frame(ToxAV *av, uint32_t friend_number, uint16_t width, u
 
     int vpx_encode_flags = 0;
 
-    if (m_friend_exists(av->m, friend_number) == 0) {
+    if (!m_friend_exists(av->m, friend_number)) {
         rc = TOXAV_ERR_SEND_FRAME_FRIEND_NOT_FOUND;
         goto RETURN;
     }
@@ -858,8 +948,8 @@ bool toxav_video_send_frame(ToxAV *av, uint32_t friend_number, uint16_t width, u
     }
 
     if (call->video_bit_rate == 0 ||
-            !(call->msi_call->self_capabilities & MSI_CAP_S_VIDEO) ||
-            !(call->msi_call->peer_capabilities & MSI_CAP_R_VIDEO)) {
+            (call->msi_call->self_capabilities & MSI_CAP_S_VIDEO) == 0 ||
+            (call->msi_call->peer_capabilities & MSI_CAP_R_VIDEO) == 0) {
         pthread_mutex_unlock(av->mutex);
         rc = TOXAV_ERR_SEND_FRAME_PAYLOAD_TYPE_DISABLED;
         goto RETURN;
@@ -883,13 +973,13 @@ bool toxav_video_send_frame(ToxAV *av, uint32_t friend_number, uint16_t width, u
     if (call->video_rtp->ssrc < VIDEO_SEND_X_KEYFRAMES_FIRST) {
         // Key frame flag for first frames
         vpx_encode_flags = VPX_EFLAG_FORCE_KF;
-        LOGGER_INFO(av->m->log, "I_FRAME_FLAG:%d only-i-frame mode", call->video_rtp->ssrc);
+        LOGGER_DEBUG(av->m->log, "I_FRAME_FLAG:%d only-i-frame mode", call->video_rtp->ssrc);
 
         ++call->video_rtp->ssrc;
     } else if (call->video_rtp->ssrc == VIDEO_SEND_X_KEYFRAMES_FIRST) {
         // normal keyframe placement
         vpx_encode_flags = 0;
-        LOGGER_INFO(av->m->log, "I_FRAME_FLAG:%d normal mode", call->video_rtp->ssrc);
+        LOGGER_DEBUG(av->m->log, "I_FRAME_FLAG:%d normal mode", call->video_rtp->ssrc);
 
         ++call->video_rtp->ssrc;
     }
@@ -911,7 +1001,7 @@ bool toxav_video_send_frame(ToxAV *av, uint32_t friend_number, uint16_t width, u
         memcpy(img.planes[VPX_PLANE_U], u, (width / 2) * (height / 2));
         memcpy(img.planes[VPX_PLANE_V], v, (width / 2) * (height / 2));
 
-        vpx_codec_err_t vrc = vpx_codec_encode(call->video->encoder, &img,
+        const vpx_codec_err_t vrc = vpx_codec_encode(call->video->encoder, &img,
                                                call->video->frame_counter, 1, vpx_encode_flags, MAX_ENCODE_TIME_US);
 
         vpx_img_free(&img);
@@ -932,7 +1022,7 @@ bool toxav_video_send_frame(ToxAV *av, uint32_t friend_number, uint16_t width, u
 
 RETURN:
 
-    if (error) {
+    if (error != nullptr) {
         *error = rc;
     }
 
@@ -970,19 +1060,19 @@ static void callback_bwc(BWController *bwc, uint32_t friend_number, float loss,
      */
 
     ToxAVCall *call = (ToxAVCall *)user_data;
-    assert(call);
+    assert(call != nullptr);
 
     LOGGER_DEBUG(call->av->m->log, "Reported loss of %f%%", (double)loss * 100);
 
     /* if less than 10% data loss we do nothing! */
-    if (loss < 0.1f) {
+    if (loss < 0.1F) {
         return;
     }
 
     pthread_mutex_lock(call->av->mutex);
 
-    if (call->video_bit_rate) {
-        if (!call->av->vbcb) {
+    if (call->video_bit_rate != 0) {
+        if (call->av->vbcb == nullptr) {
             pthread_mutex_unlock(call->av->mutex);
             LOGGER_WARNING(call->av->m->log, "No callback to report loss on");
             return;
@@ -991,8 +1081,8 @@ static void callback_bwc(BWController *bwc, uint32_t friend_number, float loss,
         call->av->vbcb(call->av, friend_number,
                        call->video_bit_rate - (call->video_bit_rate * loss),
                        call->av->vbcb_user_data);
-    } else if (call->audio_bit_rate) {
-        if (!call->av->abcb) {
+    } else if (call->audio_bit_rate != 0) {
+        if (call->av->abcb == nullptr) {
             pthread_mutex_unlock(call->av->mutex);
             LOGGER_WARNING(call->av->m->log, "No callback to report loss on");
             return;
@@ -1021,7 +1111,7 @@ static int callback_invite(void *toxav_inst, MSICall *call)
     call->av_call = av_call;
     av_call->msi_call = call;
 
-    if (toxav->ccb) {
+    if (toxav->ccb != nullptr) {
         toxav->ccb(toxav, call->friend_number, call->peer_capabilities & MSI_CAP_S_AUDIO,
                    call->peer_capabilities & MSI_CAP_S_VIDEO, toxav->ccb_user_data);
     } else {
@@ -1068,7 +1158,7 @@ static int callback_end(void *toxav_inst, MSICall *call)
 
     invoke_call_state_callback(toxav, call->friend_number, TOXAV_FRIEND_CALL_STATE_FINISHED);
 
-    if (call->av_call) {
+    if (call->av_call != nullptr) {
         call_kill_transmission(call->av_call);
         call_remove(call->av_call);
     }
@@ -1083,7 +1173,7 @@ static int callback_error(void *toxav_inst, MSICall *call)
 
     invoke_call_state_callback(toxav, call->friend_number, TOXAV_FRIEND_CALL_STATE_ERROR);
 
-    if (call->av_call) {
+    if (call->av_call != nullptr) {
         call_kill_transmission(call->av_call);
         call_remove(call->av_call);
     }
@@ -1096,13 +1186,13 @@ static int callback_capabilites(void *toxav_inst, MSICall *call)
     ToxAV *toxav = (ToxAV *)toxav_inst;
     pthread_mutex_lock(toxav->mutex);
 
-    if (call->peer_capabilities & MSI_CAP_S_AUDIO) {
+    if ((call->peer_capabilities & MSI_CAP_S_AUDIO) != 0) {
         rtp_allow_receiving(call->av_call->audio_rtp);
     } else {
         rtp_stop_receiving(call->av_call->audio_rtp);
     }
 
-    if (call->peer_capabilities & MSI_CAP_S_VIDEO) {
+    if ((call->peer_capabilities & MSI_CAP_S_VIDEO) != 0) {
         rtp_allow_receiving(call->av_call->video_rtp);
     } else {
         rtp_stop_receiving(call->av_call->video_rtp);
@@ -1129,11 +1219,11 @@ static bool video_bit_rate_invalid(uint32_t bit_rate)
      * we may want to prevent from passing overflowed bitrates to libvpx
      * more in detail, it's the case where bit_rate is larger than uint, but smaller than uint32_t
      */
-    return bit_rate > UINT_MAX;
+    return bit_rate > UINT32_MAX;
 }
 static bool invoke_call_state_callback(ToxAV *av, uint32_t friend_number, uint32_t state)
 {
-    if (av->scb) {
+    if (av->scb != nullptr) {
         av->scb(av, friend_number, state, av->scb_user_data);
     } else {
         return false;
@@ -1148,7 +1238,7 @@ static ToxAVCall *call_new(ToxAV *av, uint32_t friend_number, Toxav_Err_Call *er
     Toxav_Err_Call rc = TOXAV_ERR_CALL_OK;
     ToxAVCall *call = nullptr;
 
-    if (m_friend_exists(av->m, friend_number) == 0) {
+    if (!m_friend_exists(av->m, friend_number)) {
         rc = TOXAV_ERR_CALL_FRIEND_NOT_FOUND;
         goto RETURN;
     }
@@ -1163,7 +1253,7 @@ static ToxAVCall *call_new(ToxAV *av, uint32_t friend_number, Toxav_Err_Call *er
         goto RETURN;
     }
 
-    call = (ToxAVCall *)calloc(sizeof(ToxAVCall), 1);
+    call = (ToxAVCall *)calloc(1, sizeof(ToxAVCall));
 
     if (call == nullptr) {
         rc = TOXAV_ERR_CALL_MALLOC;
@@ -1173,7 +1263,7 @@ static ToxAVCall *call_new(ToxAV *av, uint32_t friend_number, Toxav_Err_Call *er
     call->av = av;
     call->friend_number = friend_number;
 
-    if (create_recursive_mutex(call->toxav_call_mutex)) {
+    if (create_recursive_mutex(call->toxav_call_mutex) != 0) {
         free(call);
         call = nullptr;
         rc = TOXAV_ERR_CALL_MALLOC;
@@ -1181,7 +1271,7 @@ static ToxAVCall *call_new(ToxAV *av, uint32_t friend_number, Toxav_Err_Call *er
     }
 
     if (av->calls == nullptr) { /* Creating */
-        av->calls = (ToxAVCall **)calloc(sizeof(ToxAVCall *), friend_number + 1);
+        av->calls = (ToxAVCall **)calloc(friend_number + 1, sizeof(ToxAVCall *));
 
         if (av->calls == nullptr) {
             pthread_mutex_destroy(call->toxav_call_mutex);
@@ -1225,7 +1315,7 @@ static ToxAVCall *call_new(ToxAV *av, uint32_t friend_number, Toxav_Err_Call *er
 
 RETURN:
 
-    if (error) {
+    if (error != nullptr) {
         *error = rc;
     }
 
@@ -1248,7 +1338,7 @@ static ToxAVCall *call_remove(ToxAVCall *call)
         return nullptr;
     }
 
-    uint32_t friend_number = call->friend_number;
+    const uint32_t friend_number = call->friend_number;
     ToxAV *av = call->av;
 
     ToxAVCall *prev = call->prev;
@@ -1257,24 +1347,24 @@ static ToxAVCall *call_remove(ToxAVCall *call)
     /* Set av call in msi to NULL in order to know if call if ToxAVCall is
      * removed from the msi call.
      */
-    if (call->msi_call) {
+    if (call->msi_call != nullptr) {
         call->msi_call->av_call = nullptr;
     }
 
     pthread_mutex_destroy(call->toxav_call_mutex);
     free(call);
 
-    if (prev) {
+    if (prev != nullptr) {
         prev->next = next;
-    } else if (next) {
+    } else if (next != nullptr) {
         av->calls_head = next->friend_number;
     } else {
         goto CLEAR;
     }
 
-    if (next) {
+    if (next != nullptr) {
         next->prev = prev;
-    } else if (prev) {
+    } else if (prev != nullptr) {
         av->calls_tail = prev->friend_number;
     } else {
         goto CLEAR;
@@ -1362,7 +1452,7 @@ static bool call_prepare_transmission(ToxAVCall *call)
         }
     }
 
-    call->active = 1;
+    call->active = true;
     return true;
 
 FAILURE:
@@ -1383,11 +1473,11 @@ FAILURE_2:
 
 static void call_kill_transmission(ToxAVCall *call)
 {
-    if (call == nullptr || call->active == 0) {
+    if (call == nullptr || !call->active) {
         return;
     }
 
-    call->active = 0;
+    call->active = false;
 
     pthread_mutex_lock(call->mutex_audio);
     pthread_mutex_unlock(call->mutex_audio);
