@@ -8,12 +8,12 @@
 #include <stdbool.h>
 #include <stdint.h>
 
+#include "attributes.h"
+
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-#ifndef MONO_TIME_DEFINED
-#define MONO_TIME_DEFINED
 /**
  * The timer portion of the toxcore event loop.
  *
@@ -44,40 +44,48 @@ extern "C" {
  * implementation should at least theoretically match the specification.
  */
 typedef struct Mono_Time Mono_Time;
-#endif /* MONO_TIME_DEFINED */
 
-Mono_Time *mono_time_new(void);
+typedef uint64_t mono_time_current_time_cb(void *user_data);
+
+nullable(1, 2)
+Mono_Time *mono_time_new(mono_time_current_time_cb *current_time_callback, void *user_data);
+
+nullable(1)
 void mono_time_free(Mono_Time *mono_time);
 
 /**
  * Update mono_time; subsequent calls to mono_time_get or mono_time_is_timeout
  * will use the time at the call to mono_time_update.
  */
+non_null()
 void mono_time_update(Mono_Time *mono_time);
 
 /**
  * Return unix time since epoch in seconds.
  */
+non_null()
 uint64_t mono_time_get(const Mono_Time *mono_time);
 
 /**
  * Return true iff timestamp is at least timeout seconds in the past.
  */
+non_null()
 bool mono_time_is_timeout(const Mono_Time *mono_time, uint64_t timestamp, uint64_t timeout);
 
 /**
  * Return current monotonic time in milliseconds (ms). The starting point is
  * unspecified.
  */
+non_null()
 uint64_t current_time_monotonic(Mono_Time *mono_time);
 
-typedef uint64_t mono_time_current_time_cb(Mono_Time *mono_time, void *user_data);
-
-/* Override implementation of current_time_monotonic() (for tests).
+/**
+ * Override implementation of `current_time_monotonic()` (for tests).
  *
- * The caller is obligated to ensure that current_time_monotonic() continues
+ * The caller is obligated to ensure that `current_time_monotonic()` continues
  * to increase monotonically.
  */
+non_null(1) nullable(2, 3)
 void mono_time_set_current_time_callback(Mono_Time *mono_time,
         mono_time_current_time_cb *current_time_callback, void *user_data);
 
