@@ -4,19 +4,16 @@
  * Copyright © 2013 plutooo
  */
 
-/*
+/**
  * Buffered pinging using cyclic arrays.
  */
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include "ping.h"
 
 #include <stdlib.h>
 #include <string.h>
 
 #include "DHT.h"
+#include "ccompat.h"
 #include "mono_time.h"
 #include "network.h"
 #include "ping_array.h"
@@ -24,15 +21,16 @@
 
 #define PING_NUM_MAX 512
 
-/* Maximum newly announced nodes to ping per TIME_TO_PING seconds. */
+/** Maximum newly announced nodes to ping per TIME_TO_PING seconds. */
 #define MAX_TO_PING 32
 
-/* Ping newly announced nodes to ping per TIME_TO_PING seconds*/
+/** Ping newly announced nodes to ping per TIME_TO_PING seconds*/
 #define TIME_TO_PING 2
 
 
 struct Ping {
     const Mono_Time *mono_time;
+    const Random *rng;
     DHT *dht;
 
     Ping_Array  *ping_array;
@@ -45,14 +43,14 @@ struct Ping {
 #define DHT_PING_SIZE (1 + CRYPTO_PUBLIC_KEY_SIZE + CRYPTO_NONCE_SIZE + PING_PLAIN_SIZE + CRYPTO_MAC_SIZE)
 #define PING_DATA_SIZE (CRYPTO_PUBLIC_KEY_SIZE + sizeof(IP_Port))
 
-int32_t ping_send_request(Ping *ping, IP_Port ipp, const uint8_t *public_key)
+void ping_send_request(Ping *ping, const IP_Port *ipp, const uint8_t *public_key)
 {
     uint8_t   pk[DHT_PING_SIZE];
     int       rc;
     uint64_t  ping_id;
 
-    if (id_equal(public_key, dht_get_self_public_key(ping->dht))) {
-        return 1;
+    if (pk_equal(public_key, dht_get_self_public_key(ping->dht))) {
+        return;
     }
 
     uint8_t shared_key[CRYPTO_SHARED_KEY_SIZE];
@@ -61,12 +59,13 @@ int32_t ping_send_request(Ping *ping, IP_Port ipp, const uint8_t *public_key)
     dht_get_shared_key_sent(ping->dht, shared_key, public_key);
     // Generate random ping_id.
     uint8_t data[PING_DATA_SIZE];
-    id_copy(data, public_key);
-    memcpy(data + CRYPTO_PUBLIC_KEY_SIZE, &ipp, sizeof(IP_Port));
-    ping_id = ping_array_add(ping->ping_array, ping->mono_time, data, sizeof(data));
+    pk_copy(data, public_key);
+    memcpy(data + CRYPTO_PUBLIC_KEY_SIZE, ipp, sizeof(IP_Port));
+    ping_id = ping_array_add(ping->ping_array, ping->mono_time, ping->rng, data, sizeof(data));
 
     if (ping_id == 0) {
-        return 1;
+        crypto_memzero(shared_key, sizeof(shared_key));
+        return;
     }
 
     uint8_t ping_plain[PING_PLAIN_SIZE];
@@ -74,8 +73,8 @@ int32_t ping_send_request(Ping *ping, IP_Port ipp, const uint8_t *public_key)
     memcpy(ping_plain + 1, &ping_id, sizeof(ping_id));
 
     pk[0] = NET_PACKET_PING_REQUEST;
-    id_copy(pk + 1, dht_get_self_public_key(ping->dht));     // Our pubkey
-    random_nonce(pk + 1 + CRYPTO_PUBLIC_KEY_SIZE); // Generate new nonce
+    pk_copy(pk + 1, dht_get_self_public_key(ping->dht));     // Our pubkey
+    random_nonce(ping->rng, pk + 1 + CRYPTO_PUBLIC_KEY_SIZE); // Generate new nonce
 
 
     rc = encrypt_data_symmetric(shared_key,
@@ -83,20 +82,23 @@ int32_t ping_send_request(Ping *ping, IP_Port ipp, const uint8_t *public_key)
                                 ping_plain, sizeof(ping_plain),
                                 pk + 1 + CRYPTO_PUBLIC_KEY_SIZE + CRYPTO_NONCE_SIZE);
 
+    crypto_memzero(shared_key, sizeof(shared_key));
+
     if (rc != PING_PLAIN_SIZE + CRYPTO_MAC_SIZE) {
-        return 1;
+        return;
     }
 
-    return sendpacket(dht_get_net(ping->dht), ipp, pk, sizeof(pk));
+    // We never check this return value and failures in sendpacket are already logged
+    sendpacket(dht_get_net(ping->dht), ipp, pk, sizeof(pk));
 }
 
-static int ping_send_response(Ping *ping, IP_Port ipp, const uint8_t *public_key, uint64_t ping_id,
-                              uint8_t *shared_encryption_key)
+non_null()
+static int ping_send_response(const Ping *ping, const IP_Port *ipp, const uint8_t *public_key,
+                              uint64_t ping_id, const uint8_t *shared_encryption_key)
 {
-    uint8_t   pk[DHT_PING_SIZE];
-    int       rc;
+    uint8_t pk[DHT_PING_SIZE];
 
-    if (id_equal(public_key, dht_get_self_public_key(ping->dht))) {
+    if (pk_equal(public_key, dht_get_self_public_key(ping->dht))) {
         return 1;
     }
 
@@ -105,14 +107,14 @@ static int ping_send_response(Ping *ping, IP_Port ipp, const uint8_t *public_key
     memcpy(ping_plain + 1, &ping_id, sizeof(ping_id));
 
     pk[0] = NET_PACKET_PING_RESPONSE;
-    id_copy(pk + 1, dht_get_self_public_key(ping->dht));     // Our pubkey
-    random_nonce(pk + 1 + CRYPTO_PUBLIC_KEY_SIZE); // Generate new nonce
+    pk_copy(pk + 1, dht_get_self_public_key(ping->dht));     // Our pubkey
+    random_nonce(ping->rng, pk + 1 + CRYPTO_PUBLIC_KEY_SIZE); // Generate new nonce
 
     // Encrypt ping_id using recipient privkey
-    rc = encrypt_data_symmetric(shared_encryption_key,
-                                pk + 1 + CRYPTO_PUBLIC_KEY_SIZE,
-                                ping_plain, sizeof(ping_plain),
-                                pk + 1 + CRYPTO_PUBLIC_KEY_SIZE + CRYPTO_NONCE_SIZE);
+    const int rc = encrypt_data_symmetric(shared_encryption_key,
+                                          pk + 1 + CRYPTO_PUBLIC_KEY_SIZE,
+                                          ping_plain, sizeof(ping_plain),
+                                          pk + 1 + CRYPTO_PUBLIC_KEY_SIZE + CRYPTO_NONCE_SIZE);
 
     if (rc != PING_PLAIN_SIZE + CRYPTO_MAC_SIZE) {
         return 1;
@@ -121,10 +123,11 @@ static int ping_send_response(Ping *ping, IP_Port ipp, const uint8_t *public_key
     return sendpacket(dht_get_net(ping->dht), ipp, pk, sizeof(pk));
 }
 
-static int handle_ping_request(void *object, IP_Port source, const uint8_t *packet, uint16_t length, void *userdata)
+non_null()
+static int handle_ping_request(void *object, const IP_Port *source, const uint8_t *packet, uint16_t length,
+                               void *userdata)
 {
-    DHT       *dht = (DHT *)object;
-    int        rc;
+    DHT *dht = (DHT *)object;
 
     if (length != DHT_PING_SIZE) {
         return 1;
@@ -132,39 +135,45 @@ static int handle_ping_request(void *object, IP_Port source, const uint8_t *pack
 
     Ping *ping = dht_get_ping(dht);
 
-    if (id_equal(packet + 1, dht_get_self_public_key(ping->dht))) {
+    if (pk_equal(packet + 1, dht_get_self_public_key(ping->dht))) {
         return 1;
     }
 
     uint8_t shared_key[CRYPTO_SHARED_KEY_SIZE];
-
     uint8_t ping_plain[PING_PLAIN_SIZE];
+
     // Decrypt ping_id
     dht_get_shared_key_recv(dht, shared_key, packet + 1);
-    rc = decrypt_data_symmetric(shared_key,
-                                packet + 1 + CRYPTO_PUBLIC_KEY_SIZE,
-                                packet + 1 + CRYPTO_PUBLIC_KEY_SIZE + CRYPTO_NONCE_SIZE,
-                                PING_PLAIN_SIZE + CRYPTO_MAC_SIZE,
-                                ping_plain);
+    const int rc = decrypt_data_symmetric(shared_key,
+                                          packet + 1 + CRYPTO_PUBLIC_KEY_SIZE,
+                                          packet + 1 + CRYPTO_PUBLIC_KEY_SIZE + CRYPTO_NONCE_SIZE,
+                                          PING_PLAIN_SIZE + CRYPTO_MAC_SIZE,
+                                          ping_plain);
 
     if (rc != sizeof(ping_plain)) {
+        crypto_memzero(shared_key, sizeof(shared_key));
         return 1;
     }
 
     if (ping_plain[0] != NET_PACKET_PING_REQUEST) {
+        crypto_memzero(shared_key, sizeof(shared_key));
         return 1;
     }
 
-    uint64_t   ping_id;
+    uint64_t ping_id;
     memcpy(&ping_id, ping_plain + 1, sizeof(ping_id));
     // Send response
     ping_send_response(ping, source, packet + 1, ping_id, shared_key);
     ping_add(ping, packet + 1, source);
 
+    crypto_memzero(shared_key, sizeof(shared_key));
+
     return 0;
 }
 
-static int handle_ping_response(void *object, IP_Port source, const uint8_t *packet, uint16_t length, void *userdata)
+non_null()
+static int handle_ping_response(void *object, const IP_Port *source, const uint8_t *packet, uint16_t length,
+                                void *userdata)
 {
     DHT      *dht = (DHT *)object;
     int       rc;
@@ -175,7 +184,7 @@ static int handle_ping_response(void *object, IP_Port source, const uint8_t *pac
 
     Ping *ping = dht_get_ping(dht);
 
-    if (id_equal(packet + 1, dht_get_self_public_key(ping->dht))) {
+    if (pk_equal(packet + 1, dht_get_self_public_key(ping->dht))) {
         return 1;
     }
 
@@ -192,6 +201,8 @@ static int handle_ping_response(void *object, IP_Port source, const uint8_t *pac
                                 PING_PLAIN_SIZE + CRYPTO_MAC_SIZE,
                                 ping_plain);
 
+    crypto_memzero(shared_key, sizeof(shared_key));
+
     if (rc != sizeof(ping_plain)) {
         return 1;
     }
@@ -208,14 +219,14 @@ static int handle_ping_response(void *object, IP_Port source, const uint8_t *pac
         return 1;
     }
 
-    if (!id_equal(packet + 1, data)) {
+    if (!pk_equal(packet + 1, data)) {
         return 1;
     }
 
     IP_Port ipp;
     memcpy(&ipp, data + CRYPTO_PUBLIC_KEY_SIZE, sizeof(IP_Port));
 
-    if (!ipport_equal(&ipp, &source)) {
+    if (!ipport_equal(&ipp, source)) {
         return 1;
     }
 
@@ -223,48 +234,48 @@ static int handle_ping_response(void *object, IP_Port source, const uint8_t *pac
     return 0;
 }
 
-/* Check if public_key with ip_port is in the list.
+/** @brief Check if public_key with ip_port is in the list.
  *
- * return 1 if it is.
- * return 0 if it isn't.
+ * return true if it is.
+ * return false if it isn't.
  */
-static int in_list(const Client_data *list, uint16_t length, const Mono_Time *mono_time, const uint8_t *public_key,
-                   IP_Port ip_port)
+non_null()
+static bool in_list(const Client_data *list, uint16_t length, const Mono_Time *mono_time, const uint8_t *public_key,
+                    const IP_Port *ip_port)
 {
-    unsigned int i;
-
-    for (i = 0; i < length; ++i) {
-        if (id_equal(list[i].public_key, public_key)) {
+    for (unsigned int i = 0; i < length; ++i) {
+        if (pk_equal(list[i].public_key, public_key)) {
             const IPPTsPng *ipptp;
 
-            if (net_family_is_ipv4(ip_port.ip.family)) {
+            if (net_family_is_ipv4(ip_port->ip.family)) {
                 ipptp = &list[i].assoc4;
             } else {
                 ipptp = &list[i].assoc6;
             }
 
-            if (!mono_time_is_timeout(mono_time, ipptp->timestamp, BAD_NODE_TIMEOUT) && ipport_equal(&ipptp->ip_port, &ip_port)) {
-                return 1;
+            if (!mono_time_is_timeout(mono_time, ipptp->timestamp, BAD_NODE_TIMEOUT)
+                    && ipport_equal(&ipptp->ip_port, ip_port)) {
+                return true;
             }
         }
     }
 
-    return 0;
+    return false;
 }
 
-/* Add nodes to the to_ping list.
+/** @brief Add nodes to the to_ping list.
  * All nodes in this list are pinged every TIME_TO_PING seconds
  * and are then removed from the list.
  * If the list is full the nodes farthest from our public_key are replaced.
  * The purpose of this list is to enable quick integration of new nodes into the
  * network while preventing amplification attacks.
  *
- *  return 0 if node was added.
- *  return -1 if node was not added.
+ * @retval 0 if node was added.
+ * @retval -1 if node was not added.
  */
-int32_t ping_add(Ping *ping, const uint8_t *public_key, IP_Port ip_port)
+int32_t ping_add(Ping *ping, const uint8_t *public_key, const IP_Port *ip_port)
 {
-    if (!ip_isset(&ip_port.ip)) {
+    if (!ip_isset(&ip_port->ip)) {
         return -1;
     }
 
@@ -283,16 +294,14 @@ int32_t ping_add(Ping *ping, const uint8_t *public_key, IP_Port ip_port)
         return -1;
     }
 
-    unsigned int i;
-
-    for (i = 0; i < MAX_TO_PING; ++i) {
+    for (unsigned int i = 0; i < MAX_TO_PING; ++i) {
         if (!ip_isset(&ping->to_ping[i].ip_port.ip)) {
             memcpy(ping->to_ping[i].public_key, public_key, CRYPTO_PUBLIC_KEY_SIZE);
-            ipport_copy(&ping->to_ping[i].ip_port, &ip_port);
+            ipport_copy(&ping->to_ping[i].ip_port, ip_port);
             return 0;
         }
 
-        if (public_key_cmp(ping->to_ping[i].public_key, public_key) == 0) {
+        if (pk_equal(ping->to_ping[i].public_key, public_key)) {
             return -1;
         }
     }
@@ -305,7 +314,7 @@ int32_t ping_add(Ping *ping, const uint8_t *public_key, IP_Port ip_port)
 }
 
 
-/* Ping all the valid nodes in the to_ping list every TIME_TO_PING seconds.
+/** @brief Ping all the valid nodes in the to_ping list every TIME_TO_PING seconds.
  * This function must be run at least once every TIME_TO_PING seconds.
  */
 void ping_iterate(Ping *ping)
@@ -325,11 +334,11 @@ void ping_iterate(Ping *ping)
             break;
         }
 
-        if (!node_addable_to_close_list(ping->dht, ping->to_ping[i].public_key, ping->to_ping[i].ip_port)) {
+        if (!node_addable_to_close_list(ping->dht, ping->to_ping[i].public_key, &ping->to_ping[i].ip_port)) {
             continue;
         }
 
-        ping_send_request(ping, ping->to_ping[i].ip_port, ping->to_ping[i].public_key);
+        ping_send_request(ping, &ping->to_ping[i].ip_port, ping->to_ping[i].public_key);
         ip_reset(&ping->to_ping[i].ip_port.ip);
     }
 
@@ -339,7 +348,7 @@ void ping_iterate(Ping *ping)
 }
 
 
-Ping *ping_new(const Mono_Time *mono_time, DHT *dht)
+Ping *ping_new(const Mono_Time *mono_time, const Random *rng, DHT *dht)
 {
     Ping *ping = (Ping *)calloc(1, sizeof(Ping));
 
@@ -355,6 +364,7 @@ Ping *ping_new(const Mono_Time *mono_time, DHT *dht)
     }
 
     ping->mono_time = mono_time;
+    ping->rng = rng;
     ping->dht = dht;
     networking_registerhandler(dht_get_net(ping->dht), NET_PACKET_PING_REQUEST, &handle_ping_request, dht);
     networking_registerhandler(dht_get_net(ping->dht), NET_PACKET_PING_RESPONSE, &handle_ping_response, dht);
