@@ -6,7 +6,9 @@
 
 #include <string.h>
 
-/* state load/save */
+#include "ccompat.h"
+
+/** state load/save */
 int state_load(const Logger *log, state_load_cb *state_load_callback, void *outer,
                const uint8_t *data, uint32_t length, uint16_t cookie_inner)
 {
@@ -34,7 +36,7 @@ int state_load(const Logger *log, state_load_cb *state_load_callback, void *oute
             return -1;
         }
 
-        if (lendian_to_host16((cookie_type >> 16)) != cookie_inner) {
+        if (lendian_to_host16(cookie_type >> 16) != cookie_inner) {
             /* something is not matching up in a bad way, give up */
             LOGGER_ERROR(log, "state file garbled: %04x != %04x", cookie_type >> 16, cookie_inner);
             return -1;
@@ -43,17 +45,20 @@ int state_load(const Logger *log, state_load_cb *state_load_callback, void *oute
         const uint16_t type = lendian_to_host16(cookie_type & 0xFFFF);
 
         switch (state_load_callback(outer, data, length_sub, type)) {
-            case STATE_LOAD_STATUS_CONTINUE:
+            case STATE_LOAD_STATUS_CONTINUE: {
                 data += length_sub;
                 length -= length_sub;
                 break;
+            }
 
-            case STATE_LOAD_STATUS_ERROR:
+            case STATE_LOAD_STATUS_ERROR: {
                 LOGGER_ERROR(log, "Error occcured in state file (type: %u).", type);
                 return -1;
+            }
 
-            case STATE_LOAD_STATUS_END:
+            case STATE_LOAD_STATUS_END: {
                 return 0;
+            }
         }
     }
 
