@@ -4,13 +4,9 @@
  * Copyright © 2013 plutooo
  */
 
-/*
+/**
  * Utilities.
  */
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #ifndef _XOPEN_SOURCE
 #define _XOPEN_SOURCE 600
 #endif
@@ -21,19 +17,39 @@
 #include <string.h>
 #include <time.h>
 
-#include "crypto_core.h" /* for CRYPTO_PUBLIC_KEY_SIZE */
+#include "ccompat.h"
 
-
-/* id functions */
-bool id_equal(const uint8_t *dest, const uint8_t *src)
+bool is_power_of_2(uint64_t x)
 {
-    return public_key_cmp(dest, src) == 0;
+    return x != 0 && (x & (~x + 1)) == x;
 }
 
-uint32_t id_copy(uint8_t *dest, const uint8_t *src)
+void free_uint8_t_pointer_array(uint8_t **ary, size_t n_items)
 {
-    memcpy(dest, src, CRYPTO_PUBLIC_KEY_SIZE);
-    return CRYPTO_PUBLIC_KEY_SIZE;
+    if (ary == nullptr) {
+        return;
+    }
+
+    for (size_t i = 0; i < n_items; ++i) {
+        if (ary[i] != nullptr) {
+            free(ary[i]);
+        }
+    }
+
+    free(ary);
+}
+
+uint16_t data_checksum(const uint8_t *data, uint32_t length)
+{
+    uint8_t checksum[2] = {0};
+    uint16_t check;
+
+    for (uint32_t i = 0; i < length; ++i) {
+        checksum[i % 2] ^= data[i];
+    }
+
+    memcpy(&check, checksum, sizeof(check));
+    return check;
 }
 
 int create_recursive_mutex(pthread_mutex_t *mutex)
@@ -60,6 +76,11 @@ int create_recursive_mutex(pthread_mutex_t *mutex)
     return 0;
 }
 
+bool memeq(const uint8_t *a, size_t a_size, const uint8_t *b, size_t b_size)
+{
+    return a_size == b_size && memcmp(a, b, a_size) == 0;
+}
+
 int16_t max_s16(int16_t a, int16_t b)
 {
     return a > b ? a : b;
@@ -111,3 +132,19 @@ uint64_t min_u64(uint64_t a, uint64_t b)
 {
     return a < b ? a : b;
 }
+
+uint32_t jenkins_one_at_a_time_hash(const uint8_t *key, size_t len)
+{
+    uint32_t hash = 0;
+
+    for (uint32_t i = 0; i < len; ++i) {
+        hash += key[i];
+        hash += hash << 10;
+        hash ^= hash >> 6;
+    }
+
+    hash += hash << 3;
+    hash ^= hash >> 11;
+    hash += hash << 15;
+    return hash;
+}
